from twisted.python.deprecate import deprecatedModuleAttribute
from twisted.python.versions import Version

from axiom import attributes

from nevow.util import Expose

from methanal import errors
from methanal.util import propertyMaker



constraint = Expose(
    """
    Register one or more functions as constraints for a particular form
    parameter.
    """)



class Value(object):
    """
    A simple value in a model.

    @type name: C{str}
    @ivar name: Name of this parameter

    @ivar value: Initial value of this parameter

    @type doc: C{unicode}
    @ivar doc: A long description of this parameter
    """
    def __init__(self, name, value=None, doc=None, **kw):
        """
        Initialise the parameter.

        @type name: C{str}
        @ivar name: Name of this parameter

        @ivar value: Initial value of this parameter

        @type doc: C{unicode}
        @ivar doc: A long description of this parameter, or C{None} to use the
            parameter's name
        """
        super(Value, self).__init__(**kw)
        self.name = name
        self.value = value
        if doc is None:
            doc = name.decode('ascii')
        self.doc = doc


    def __repr__(self):
        return '<%s name=%r value=%r doc=%r>' % (
            type(self).__name__,
            self.name,
            self.value,
            self.doc)


    @classmethod
    def fromAttr(cls, attr, **kw):
        """
        Construct a parameter from an Axiom attribute.
        """
        kw.setdefault('name', attr.attrname)
        kw.setdefault('value', attr.default)
        kw.setdefault('doc', attr.doc or None)
        return cls(**kw)


    def validate(self, value):
        """
        Validate a value provided for this parameter against all constraints.

        If any constraints were violated a descriptive message is returned,
        otherwise C{None} is returned upon successful validation.
        """
        for constraintName in constraint.exposedMethodNames(self):
            result = constraint.get(self, constraintName)(value)
            if result is not None:
                return result


    def isValid(self, value):
        """
        Determine whether C{value} is valid data for this parameter.
        """
        return self.validate(value) is None


    def getValue(self):
        """
        Retrieve the value for this parameter.

        @raise errors.ConstraintError: If validation for this parameter's value
            failed
        """
        error = self.validate(self.value)
        if error:
            raise errors.ConstraintError(error)
        return self.value



class List(Value):
    """
    A model parameter consisting of multiple values.
    """
    @constraint
    def isIterable(self, value):
        """
        Enforce the iterable constraint.
        """
        try:
            iter(value)
        except TypeError:
            if value is not None:
                return u'Value is not an iterable'



class Enum(Value):
    """
    An enumeration value in a model.

    The value for this parameter must be chosen from a sequence of predefined
    possibilities.

    @ivar values: A sequence of values present in the enumeration
    """
    def __init__(self, values, **kw):
        super(Enum, self).__init__(**kw)
        self.values = values


    @constraint
    def valueInEnumeration(self, value):
        """
        Enforce the enumeration constraint.
        """
        if value not in self.values:
            return u'Value not present in enumeration'



class MultiEnum(Enum):
    """
    A multi-value enumeration model parameter.
    """
    @constraint
    def valueInEnumeration(self, value):
        """
        Enforce the enumeration constraint.
        """
        for v in value:
            if v not in self.values:
                return u'Value not present in enumeration'



class ReferenceParameter(Value):
    """
    XXX: hax :<

    Really, don't use this unless you know what you are doing.
    """
    def __init__(self, model, **kw):
        self.model = model
        super(ReferenceParameter, self).__init__(**kw)


    @classmethod
    def fromAttr(cls, attr, model, **kw):
        return super(ReferenceParameter, cls).fromAttr(attr, model=model)


    @propertyMaker
    def value():
        def get(self):
            return self.model.item
        def set(self, data):
            self.model.item = data
        return get, set


    def getValue(self):
        return self.model.process()



class DecimalValue(Value):
    """
    A decimal number model parameter.
    """
    def __init__(self, decimalPlaces, **kw):
        super(DecimalValue, self).__init__(**kw)
        self.decimalPlaces = decimalPlaces


    @classmethod
    def fromAttr(cls, attr, **kw):
        kw.setdefault('decimalPlaces', attr.decimalPlaces)
        return super(DecimalValue, cls).fromAttr(attr, **kw)



class ForeignRef(Value):
    """
    XXX: Describe me.
    """
    def __init__(self, itemType, store, **kw):
        super(ForeignRef, self).__init__(**kw)
        self.itemType = itemType
        self.store = store

    @classmethod
    def fromAttr(cls, attr, store, **kw):
        return super(ForeignRef, cls).fromAttr(
            attr, itemType=attr.reftype, store=store, **kw)



def mandatory(value):
    """
    An external constraint that prohibits a value of C{None}.
    """
    if value is None:
        return u'Value is mandatory'



class Model(object):
    """
    A Methanal form model.
    """
    def __init__(self, params=[], callback=lambda **d: d, doc=u''):
        """
        Initialise the model.

        @type params: C{iterable} of parameter instances
        @param params: Model parameters

        @type callback: C{callable} taking parameters named the same as the
            model parameters

        @type doc: C{unicode}
        @param doc: A description for the model's action
        """
        self.params = {}
        self.attach(*params)

        self.callback = callback
        self.doc = doc


    def attach(self, *params):
        """
        Attach parameters to this model.

        @param *params: The parameters to attach.
        """
        for param in params:
            self.params[param.name] = param


    def process(self):
        """
        Invoke L{self.callback} with all the model parameter data.
        """
        data = self.getData()
        return self.callback(**data)


    def getData(self):
        """
        Get all of the model parameters' values.

        @rtype: C{dict} mapping C{str} to values
        """
        data = {}
        for param in self.params.itervalues():
            data[param.name] = param.getValue()
        return data



_paramTypes = {
    attributes.integer:        Value,
    attributes.ieee754_double: Value,
    attributes.text:           Value,
    attributes.boolean:        Value,
    attributes.textlist:       List,
    attributes.timestamp:      Value}

def paramFromAttribute(store, attr, value, name=None):
    doc = attr.doc or None

    if name is None:
        name = attr.attrname

    if isinstance(attr, attributes.reference):
        if attr.reftype is None:
            raise ValueError('%r has no reference type' % (attr,))
        model = ItemModel(item=value, itemClass=attr.reftype, store=store)
        return ReferenceParameter(name=name,
                                  value=value,
                                  doc=doc,
                                  model=model)
    elif isinstance(attr, attributes.AbstractFixedPointDecimal):
        return DecimalValue(name=name,
                            value=value,
                            doc=doc,
                            decimalPlaces=attr.decimalPlaces)
    else:
        factory = _paramTypes.get(type(attr))
        if factory:
            return factory(name=name,
                           value=value,
                           doc=doc)



def paramsFromSchema(store, itemClass, item=None, ignoredAttributes=set()):
    """
    Construct model parameters from an Axiom item schema.

    @type store: L{axiom.store.Store}
    @param store: Store used for reference attributes in the item schema

    @type itemClass: C{type}
    @param itemClass: Item type whose schema model parameters are based on

    @type item: L{axiom.item.Item}
    @param item: Item instance used for retrieving attribute values, or
        C{None} to use the attribute's default

    @type ignoredAttributes: C{set} of C{str}
    @param ignoredAttributes: Names of attributes to skip creating parameters
        for

    @rtype: C{iterable} of model parameters
    """
    for name, attr in itemClass.getSchema():
        if name in ignoredAttributes:
            continue

        if item is not None:
            value = getattr(item, name)
        else:
            value = attr.default

        yield paramFromAttribute(store, attr, value, name)



class ItemModelBase(Model):
    """
    A model backed by an item or item class.
    """
    def __init__(self, item=None, itemClass=None, store=None, doc=u'Save',
                 **kw):
        """
        Initialise model.

        Either C{item} or C{itemClass} must be specified, specifying only
        an item class will result in an item of that type being created
        when the model's callback is triggered.  This can be useful for
        providing a model for an item type that you wish to create but
        have no instance of yet.

        @type item: C{axiom.item.Item}
        @param item: An Axiom item to synthesize a model for, or C{None}
            to create an item of L{itemClass} when the callback is triggered

        @type itemClass: C{type}
        @param itemClass: Item type to synthesize a model and create an
            instance for, or C{None} to use the type of L{item}

        @type store: C{axiom.store.Store}
        """
        super(ItemModelBase, self).__init__(
            callback=self.storeData, doc=doc, **kw)

        if item is None:
            if itemClass is None:
                raise ValueError('You must pass in either item or itemClass')

            self.item = None
            self.itemClass = itemClass
        else:
            self.item = item

        self.itemClass = itemClass or type(item)
        self.store = store or item.store


    def _storeData(self, data):
        """
        Store model data.
        """
        if self.item is None:
            self.item = self.itemClass(store=self.store, **data)
            self.createItem(self.item)
        else:
            for name, value in data.iteritems():
                setattr(self.item, name, value)
        return self.stored(self.item)


    def storeData(self, **data):
        """
        Model callback.

        Write model parameter values back to our item, creating a new one if
        no item instance was given.

        @rtype: C{axiom.item.Item}
        @return: The newly modified or created item
        """

        if self.store is not None:
            return self.store.transact(self._storeData, data)
        else:
            return self._storeData(data)


    def createItem(self, item):
        """
        Callback for item creation.

        @param item: The newly-created item.
        """


    def stored(self, item):
        """
        Callback for saved item.

        @param item: The modified or newly-created item.
        """



class ItemModel(ItemModelBase):
    """
    A model automatically synthesized from an Item class or instance.
    """
    def __init__(self, item=None, itemClass=None, store=None,
                 ignoredAttributes=None, **kw):
        """
        @type ignoredAttibutes: C{container}
        @param ignoredAttributes: Attribute names to ignore when synthesizing
            the model
        """
        super(ItemModel, self).__init__(
            item=item, itemClass=itemClass, store=store, params=[], **kw)

        if ignoredAttributes is None:
            ignoredAttributes = set()
        self.ignoredAttributes = ignoredAttributes

        params = paramsFromSchema(
            self.store, self.itemClass, self.item, self.ignoredAttributes)
        self.attach(*params)


    def stored(self, item):
        return item



def modelFromItem(item, ignoredAttributes=None):
    """
    Automatically synthesize a model from an Item instance.
    """
    return ItemModel(item=item, ignoredAttributes=ignoredAttributes)



def modelFromItemClass(itemClass, store=None, ignoredAttributes=None):
    """
    Automatically synthesize a model from an Item class.
    """
    return ItemModel(itemClass=itemClass,
                     store=store,
                     ignoredAttributes=ignoredAttributes)



def loadFromItem(model, item):
    """
    Initialise model parameters from an Axiom item instance.

    @type model: L{Model}

    @type item: L{axiom.item.Item}
    """
    for name, param in model.params.iteritems():
        param.value = getattr(item, name)



ValueParameter = Value
deprecatedModuleAttribute(
    Version('methanal', 0, 2, 0),
    'use methanal.model.Value instead', __name__, 'ValueParameter')

ListParameter = List
deprecatedModuleAttribute(
    Version('methanal', 0, 2, 0),
    'use methanal.model.List instead', __name__, 'ListParameter')

EnumerationParameter = Enum
deprecatedModuleAttribute(
    Version('methanal', 0, 2, 0),
    'use methanal.model.Enum instead', __name__, 'EnumerationParameter')

DecimalParameter = DecimalValue
deprecatedModuleAttribute(
    Version('methanal', 0, 2, 0),
    'use methanal.model.DecimalValue instead', __name__, 'DecimalParameter')

StoreIDParameter = ForeignRef
deprecatedModuleAttribute(
    Version('methanal', 0, 2, 0),
    'use methanal.model.ForeignRefinstead', __name__, 'StoreIDParameter')

MultiEnumerationParameter = MultiEnum
deprecatedModuleAttribute(
    Version('methanal', 0, 2, 0),
    'use methanal.model.MultiEnum', __name__, 'MultiEnumerationParameter')



from twisted.trial import unittest

from methanal import errors



class DeprecationWarningsTests(unittest.TestCase):
    """
    Tests for L{methanal.errors} module deprecation warnings.
    """
    def test_invalidEnumItem(self):
        """
        L{methanal.errors.InvalidEnumItem} is deprecated.
        """
        errors.InvalidEnumItem
        warnings = self.flushWarnings([self.test_invalidEnumItem])
        self.assertEqual(len(warnings), 1)
        self.assertEqual(warnings[0]['category'], DeprecationWarning)
        self.assertIn(
            'methanal.errors.InvalidEnumItem was deprecated in Methanal 0.4.0',
            warnings[0]['message'])



from nevow import tags

from xmantissa import webtheme



class Theme(webtheme.XHTMLDirectoryTheme):
    def head(self, request, website):
        def styleSheetLink(href):
            return tags.link(rel='stylesheet', type='text/css', href=href)

        root = website.rootURL(request)
        styles = root.child('static').child('Methanal').child('styles')

        yield styleSheetLink(styles.child('methanal.css'))
        yield (tags.xml(u'<!--[if IE 6]>'),
               styleSheetLink(styles.child('methanal-ie6.css')),
               tags.xml(u'<![endif]-->'))
        yield (tags.xml(u'<!--[if IE 7]>'),
               styleSheetLink(styles.child('methanal-ie7.css')),
               tags.xml(u'<![endif]-->'))



from zope.interface import implements

from twisted.python import log
from twisted.python.failure import Failure

from methanal.imethanal import ITextFormatter



class MethodWrapper(object):
    """
    Wrapper around a class method.

    @type unboundMethod: C{callable}
    @ivar unboundMethod: Unbound method

    @type cls: C{type}
    @ivar cls: Type that L{unboundMethod} is a method of

    @type instance: C{instance}
    @ivar instance: Instance to invoke L{unboundMethod} with
    """
    def __init__(self, unboundMethod, cls, instance):
        self.unboundMethod = unboundMethod
        self.cls = cls
        self.instance = instance


    def __call__(self, *a, **kw):
        return self.unboundMethod(self.instance, *a, **kw)



def collectMethods(inst, methodName):
    """
    Traverse an object's MRO, collecting methods.

    Collected methods are wrapped with L{MethodWrapper}.

    @type inst: C{type}
    @param inst: Type instance whose L{methodName} methods should be collected

    @type methodName: C{str}
    @param methodName: Name of the method, along the MRO, to collect

    @rtype: C{iterable} of L{MethodWrapper} instances
    @return: Wrapped methods named L{methodName} along L{inst}'s MRO
    """
    for cls in type(inst).__mro__:
        try:
            method = cls.__dict__[methodName]
        except KeyError:
            pass
        else:
            yield MethodWrapper(method, cls, inst)



def getArgsDict(inst):
    """
    Collect arguments along a class hierarchy.

    Arguments are collected by traversing the MRO in reverse order, looking
    for C{getArgs} methods, which should return a single C{dict} mapping
    C{unicode} keys to values.

    @type inst: C{type}
    @param inst: Type instance whose C{getArgs} results should be collected

    @raise ValueError: If a key is specified more than once
    @raise TypeError: If a key is not a C{unicode} instance

    @rtype: C{dict}
    @return: A dictionary combining the results of all C{getArgs} methods
        in C{inst}'s class hierarchy
    """
    args = {}
    for getter in collectMethods(inst, 'getArgs'):
        for key, value in getter().iteritems():
            if key in args:
                raise ValueError(
                    'Argument %r from %r already specified' % (key, getter.cls))
            if not isinstance(key, unicode):
                raise TypeError(
                    'Argument name %r is not unicode' % (key,))
            args[key] = value
    return args



class Porthole(object):
    """
    Observable event source.

    A porthole is the link between event emitters, and event observers. Any
    event (which are arbitrary objects) emitted will be broadcast to all
    observers registered with the porthole at that point in time.

    @type observers: C{list} of C{callable}
    """
    def __init__(self):
        self.observers = []


    def addObserver(self, observer):
        """
        Add a observer.

        This is a callable that will be invoked when a response is received for
        a finance application submission.

        @type observer: C{callable} taking one parameter
        @param observer: A callable, that participates in event broadcasting,
            taking one argument: C{event}

        @rtype: C{callable} taking no parameters
        @return: A callable that will remove L{observer} from the Porthole's
            observers
        """
        self.observers.append(observer)
        return lambda: self.observers.remove(observer)


    def emitEvent(self, event):
        """
        Emit an event to all attached observers.

        @param event: Object, representing event information, to pass to all
            attached observers
        """
        if isinstance(event, Failure):
            log.err(event)

        # Copy the list so that observers mutating the list won't wreak havoc.
        for observer in list(self.observers):
            observer(event)



def propertyMaker(func):
    """
    Create a property from C{func}'s return values.

    @type func: C{callable} returning an iterable
    @param func: Callable whose return values are passed on as positional
        arguments to C{property}

    @rtype: C{property} instance
    """
    return property(*func())



class DecimalFormatter(object):
    """
    Format values as decimal values.

    @type grouping: C{list} of C{int}
    @ivar grouping: Digit group sizes from right to left. Use C{0} to indicate
        that the previous group size should be used for the rest of the digit
        groups; use C{-1} to indicate that no more grouping should occur.
        Defaults to C{[3, 0]}.

    @type thousandsSeparator: C{unicode}
    @ivar thousandsSeparator: Separator between digit groups. Defaults to
        C{','}.

    @type decimalSeparator: C{unicode}
    @ivar decimalSeparator: Decimal separator. Defaults to C{'.'}.
    """
    implements(ITextFormatter)


    def __init__(self, grouping=[3, 0], thousandsSeparator=u',',
                 decimalSeparator=u'.'):
        self.grouping = grouping
        self.thousandsSeparator = thousandsSeparator
        self.decimalSeparator = decimalSeparator


    def groupings(self):
        """
        Generator of interpreted grouping values.
        """
        lastCount = 0
        for count in self.grouping:
            if count == 0:
                while True:
                    yield lastCount
            elif count == -1:
                return
            else:
                yield count
            lastCount = count


    def format(self, value):
        """
        Format C{value} as a decimal value, grouping digits if required.

        @rtype: C{unicode}
        """
        if not value:
            return u''

        parts = unicode(value).rsplit(self.decimalSeparator, 1)
        value = list(parts[0])
        i = len(value)
        for grouping in self.groupings():
            i -= grouping
            if i < 1:
                break
            value.insert(i, self.thousandsSeparator)

        parts[0] = u''.join(value)
        return self.decimalSeparator.join(parts)



class CurrencyFormatter(DecimalFormatter):
    """
    Format values as currency values.

    @type symbol: C{unicode}
    @ivar symbol: Currency symbol.

    @type symbolSeparator: C{unicode}
    @ivar symbolSeparator: Separator between then currency symbol and decimal
        value, defaults to C{' '}.
    """
    implements(ITextFormatter)


    def __init__(self, symbol, symbolSeparator=u' ', **kw):
        super(CurrencyFormatter, self).__init__(**kw)
        self.symbol = symbol
        self.symbolSeparator = symbolSeparator


    def format(self, value):
        """
        Format C{value} as a currency, prefixing the grouped decimal value with
        the currency symbol and separator.

        @rtype: C{unicode}
        """
        value = super(CurrencyFormatter, self).format(value)
        return self.symbol + self.symbolSeparator + value



from twisted.trial.unittest import TestCase
from twisted.python.versions import Version
from twisted.python.deprecate import _getDeprecationWarningString
from twisted.python.deprecate import DEPRECATION_WARNING_FORMAT


from axiom.store import Store
from axiom.item import Item
from axiom.attributes import integer, text, textlist, reference
from axiom.dependency import installOn

from xmantissa.website import WebSite

from methanal import errors, model as mmodel
from methanal.model import (Model, ItemModel, constraint, Value, Enum, List,
    loadFromItem)
from methanal.view import (LiveForm, FormGroup, ItemView, GroupInput,
    IntegerInput)



_marker = object()



class MethanalTests(TestCase):
    def test_constraints(self):
        def _constraint(value):
            if value != 5:
                return u'Value must be 5'

        param = Value(name='param')
        param._constraint = constraint(_constraint)
        self.assertFalse(param.isValid(4))
        self.assertTrue(param.isValid(5))

        model = Model([param])
        param.value = 3
        self.assertRaises(errors.ConstraintError, model.process)


    def test_enumeration(self):
        param = Enum(name='param', values=range(5))
        self.assertTrue(param.isValid(3))
        self.assertFalse(param.isValid(10))


    def test_processing(self):
        model = Model(
            params=[
                Value(name='foo', value=4),
                Value(name='bar', value=u'quux')])
        result = model.process()
        self.assertEquals(result, dict(foo=4, bar=u'quux'))


    def test_repr(self):
        """
        L{methanal.model.Value} has a useful human-readable representation.
        """
        self.assertEquals(
            repr(Value(name='foo', value=u'bar')),
            "<Value name='foo' value=u'bar' doc=u'foo'>")
        self.assertEquals(
            repr(Value(name='foo', value=u'bar', doc=u'Hello')),
            "<Value name='foo' value=u'bar' doc=u'Hello'>")



class ParameterTests(TestCase):
    def test_enumerationValidation(self):
        param = List(name='foo')
        self.assertTrue(param.isValid([]))
        self.assertTrue(param.isValid(None))
        self.assertFalse(param.isValid(5))



class _DummyItem(Item):
    i = integer(default=5)
    t = text(doc=u'param t')
    tl = textlist(doc=u'param tl')



class _DummyChildItem(Item):
    i = integer(default=5)



class _DummyParentItem(Item):
    r = reference(reftype=_DummyChildItem, doc=u'dummy reference')



class _BrokenReference(Item):
    r = reference()



class AttributeTests(TestCase):
    def test_valueParamNoDoc(self):
        param = Value.fromAttr(_DummyItem.i)
        self.assertIdentical(type(param), Value)
        self.assertEqual(param.name, 'i')
        self.assertEqual(param.value, 5)
        self.assertEqual(param.doc, 'i')


    def test_valueParam(self):
        param = Value.fromAttr(_DummyItem.t)
        self.assertIdentical(type(param), Value)
        self.assertEqual(param.name, 't')
        self.assertEqual(param.value, None)
        self.assertEqual(param.doc, 'param t')


    def test_listParam(self):
        param = List.fromAttr(_DummyItem.tl)
        self.assertIdentical(type(param), List)
        self.assertEqual(param.name, 'tl')
        self.assertEqual(param.value, None)
        self.assertEqual(param.doc, 'param tl')



class ItemUtilityTests(TestCase):
    def test_loadFromItem(self):
        item = _DummyItem(i=55, t=u'lulz')
        model = Model(params=[Value.fromAttr(_DummyItem.i),
                              Value.fromAttr(_DummyItem.t)])
        loadFromItem(model, item)
        self.assertEqual(model.params['i'].value, item.i)
        self.assertEqual(model.params['t'].value, item.t)



class AutoSchemaTests(TestCase):
    expectedParams = {
        'i': (Value(name='i', doc=u'i'), 5, 5),
        't': (Value(name='t', doc=u'param t'), None, u'text'),
        'tl': (List(name='tl', doc=u'param tl'), None, [u'text1', u'text2']),
        }


    def setUp(self):
        self.store = Store()


    def test_schemaAnalysis(self):
        """
        Test that parameters are correctly synthesized from an Item schema.
        """
        store = Store()
        model = ItemModel(itemClass=_DummyItem, store=store)
        params = model.params

        self.assertEquals(params.keys(), self.expectedParams.keys())
        for k in params:
            p1 = params[k]
            p2, classDefault, itemDefault = self.expectedParams[k]
            self.assertIdentical(type(p1), type(p2))
            self.assertEquals(p1.name, p2.name)
            self.assertEquals(p1.value, classDefault)
            self.assertEquals(p1.doc, p2.doc)


    def test_schemaIgnore(self):
        """
        Test that ignoredAttributes is respected.
        """
        store = Store()
        model = ItemModel(
            itemClass=_DummyItem, store=store, ignoredAttributes=set(['tl']))
        params = model.params
        self.assertNotIn('tl', params)


    def test_itemAnalysis(self):
        dummyItem = _DummyItem(
            store=self.store, i=5, t=u'text', tl=[u'text1', u'text2'])
        model = ItemModel(item=dummyItem)
        params = model.params
        for k in params:
            p1 = params[k]
            p2, classDefault, itemDefault = self.expectedParams[k]
            self.assertEquals(p1.value, itemDefault)


    def test_itemCreation(self):
        model = ItemModel(itemClass=_DummyItem, store=self.store)
        model.params['i'].value = 7
        model.params['t'].value = u'foo'

        self.assertIdentical(model.item, None)

        model.process()

        self.assertEquals(model.item.i, 7)
        self.assertEquals(model.item.t, u'foo')


    def test_itemEditing(self):
        model = ItemModel(item=_DummyItem(store=self.store))
        model.params['i'].value = 7
        model.params['t'].value = u'foo'

        model.process()

        self.assertEquals(model.item.i, 7)
        self.assertEquals(model.item.t, u'foo')


    def test_referenceAttributeCreating(self):
        dummyParent = _DummyParentItem(store=self.store, r=None)
        model = ItemModel(dummyParent)
        self.assertIdentical(dummyParent.r, None)

        model.process()
        self.assertEquals(dummyParent.r.i, 5)


    def test_referenceAttributeEditing(self):
        dummyChild = _DummyChildItem(store=self.store, i=5)
        dummyParent = _DummyParentItem(store=self.store, r=dummyChild)
        model = ItemModel(dummyParent)
        self.assertIdentical(dummyParent.r, dummyChild)

        model.params['r'].model.params['i'].value = 6
        model.process()
        self.assertIdentical(dummyParent.r, dummyChild)
        self.assertEquals(dummyChild.i, 6)


    def test_noRefType(self):
        """
        Attempting to automatically synthesise a model for an Item with a
        C{reference} attribute with no C{'reftype'} raises C{ValueError}.
        """
        brokenParent = _BrokenReference(store=self.store)
        self.assertRaises(ValueError, ItemModel, brokenParent)



class _DummyControl(object):
    invoked = 0


    def __init__(self, parent):
        parent.addFormChild(self)


    def invoke(self, data):
        self.invoked += 1



class LiveFormTests(TestCase):
    def setUp(self):
        s = self.store = Store()
        installOn(WebSite(store=s), s)

        self.model = Model(
            params=[
                Value(name='foo', value=4),
                Value(name='bar', value=u'quux')])


    def test_process(self):
        view = LiveForm(self.store, self.model)
        control = _DummyControl(view)
        view.invoke({})
        self.assertEquals(control.invoked, 1)


    def test_groups(self):
        view = LiveForm(self.store, self.model)
        group = FormGroup(view)
        control = _DummyControl(group)
        view.invoke({})
        self.assertEquals(control.invoked, 1)



class _DummyParameter(object):
    name = u'DUMMY_PARAMETER'
    value = u'DUMMY_PARAMETER_VALUE'
    doc = ''



class _DummyLiveForm(object):
    page = None
    liveFragmentChildren = []
    model = Model(params=[])


    def addFormChild(self, *args):
        pass


    def getParameter(self, name):
        return _DummyParameter()



class _GroupTestView(ItemView):
    def __init__(self, *args, **kw):
        super(_GroupTestView, self).__init__(*args, **kw)
        group = GroupInput(parent=self, name='r')
        IntegerInput(parent=group, name='i')



class GroupInputTests(TestCase):
    def setUp(self):
        self.store = Store()
        installOn(WebSite(store=self.store), self.store)


    def test_editing(self):
        dummyChild = _DummyChildItem(store=self.store)
        dummyParent = _DummyParentItem(store=self.store, r=dummyChild)
        view = _GroupTestView(item=dummyParent)
        self.assertIdentical(dummyParent.r, dummyChild)

        view.invoke({u'r': {u'i': 6}})

        self.assertIdentical(dummyParent.r, dummyChild)
        self.assertEquals(dummyChild.i, 6)


    def test_creation(self):
        dummyParent = _DummyParentItem(store=self.store, r=None)
        view = _GroupTestView(item=dummyParent, switchInPlace=True)
        self.assertIdentical(dummyParent.r, None)

        view.invoke({u'r': {u'i': 6}})

        self.assertEquals(dummyParent.r.i, 6)



class ModelTests(TestCase):
    def test_attach(self):
        model = Model()
        p = Value(name='foo')
        model.attach(p)
        self.assertIdentical(model.params['foo'], p)


    def test_attachMany(self):
        model = Model()
        p1 = Value(name='foo')
        p2 = Value(name='bar')

        model.attach(p1, p2)
        self.assertIdentical(model.params['foo'], p1)
        self.assertIdentical(model.params['bar'], p2)



class DeprecatedAttributesTests(TestCase):
    """
    Tests for deprecated attributes in L{methanal.model}.
    """
    version020 = Version('methanal', 0, 2, 0)


    def _getWarningString(self, obj, name, version):
        """
        Create the warning string used by deprecated attributes.
        """
        return _getDeprecationWarningString(
            obj.__name__ + '.' + name,
            version,
            DEPRECATION_WARNING_FORMAT + ': ')


    def assertDeprecated(self, obj, name, version):
        """
        Assert that the attribute C{name} on C{obj} was deprecated in
        C{version}, by testing whether a deprecation warning was issued.
        """
        getattr(obj, name)
        warningsShown = self.flushWarnings([
            self.assertDeprecated])
        self.assertEquals(len(warningsShown), 1)
        self.assertIdentical(warningsShown[0]['category'], DeprecationWarning)
        self.assertIn(self._getWarningString(obj, name, version),
                      warningsShown[0]['message'])


    def test_valueParameter(self):
        """
        L{methanal.model.ValueParameter} is deprecated.
        """
        self.assertDeprecated(mmodel, 'ValueParameter', self.version020)


    def test_listParameter(self):
        """
        L{methanal.model.ListParameter} is deprecated.
        """
        self.assertDeprecated(mmodel, 'ListParameter', self.version020)


    def test_enumerationParameter(self):
        """
        L{methanal.model.EnumerationParameter} is deprecated.
        """
        self.assertDeprecated(mmodel, 'EnumerationParameter', self.version020)


    def test_decimalParameter(self):
        """
        L{methanal.model.DecimalParameter} is deprecated.
        """
        self.assertDeprecated(mmodel, 'DecimalParameter', self.version020)


    def test_storeIDParameter(self):
        """
        L{methanal.model.StoreIDParameter} is deprecated.
        """
        self.assertDeprecated(mmodel, 'StoreIDParameter', self.version020)


    def test_multiEnumerationParameter(self):
        """
        L{methanal.model.MultiEnumerationParameter} is deprecated.
        """
        self.assertDeprecated(mmodel, 'MultiEnumerationParameter', self.version020)



from decimal import Decimal

from twisted.trial.unittest import TestCase

from methanal.util import (
    collectMethods, getArgsDict, Porthole, CurrencyFormatter, DecimalFormatter)



class MethodCollectorTests(TestCase):
    def test_simpleUsage(self):
        class A(object):
            def foo(self):
                return 'A'

        class B(A):
            def foo(self):
                return 'B'

        results = [m() for m in collectMethods(B(), 'foo')]
        self.assertEquals(results, ['B', 'A'])


    def test_missingMethod(self):
        class A(object):
            def foo(self):
                return 'A'

        class B(A):
            pass

        class C(B):
            def foo(self):
                return 'C'

        results = [m() for m in collectMethods(C(), 'foo')]
        self.assertEquals(results, ['C', 'A'])


    def test_noneAttribute(self):
        """
        Test that None attributes named the same as the method we are
        collecting are not silently ignored.
        """
        class A(object):
            foo = None

        class B(A):
            pass

        results1 = list(collectMethods(A(), 'foo'))
        self.assertEquals(len(results1), 1)

        results2 = list(collectMethods(B(), 'foo'))
        self.assertEquals(len(results2), 1)

        self.assertRaises(TypeError, results2[0])


    def test_argumentCollection(self):
        class A(object):
            """
            Test class with correct getArgs implementation.
            """
            def getArgs(self):
                return {u'foo': 1}

        class B(A):
            """
            Another test getArgs implementor.
            """
            def getArgs(self):
                return {u'bar': 2}

        self.assertEquals(getArgsDict(A()), {u'foo': 1})
        self.assertEquals(getArgsDict(B()), {u'foo': 1, u'bar': 2})


    def test_argumentCollision(self):
        class A(object):
            """
            Test class with correct getArgs implementation.
            """
            def getArgs(self):
                return {u'foo': 1}

        class B(A):
            """
            A getArgs implementor with a colliding argument name.
            """
            def getArgs(self):
                return {u'foo': 2}

        self.assertRaises(ValueError, getArgsDict, B())


    def test_argumentNameType(self):
        class A(object):
            """
            A getArgs implementor with an argument name of the wrong type.
            """
            def getArgs(self):
                return {'foo': 1}

        self.assertRaises(TypeError, getArgsDict, A())



class PortholeTests(TestCase):
    """
    Tests for L{Porthole}.
    """
    def setUp(self):
        """
        Set up a basic test porthole.
        """
        self.emitter = Porthole()


    def test_noObservers(self):
        """
        Emitting an event when there are no observers does nothing.
        """
        self.emitter.emitEvent(u'anEvent')
        self.assertEqual(self.emitter.observers, [])


    def test_addObserver(self):
        """
        Invoking C{addObserver} adds an observer to the list, and returns a
        callable that can be used to remove the observer again.
        """
        r = self.emitter.addObserver(None)
        self.assertEqual(self.emitter.observers, [None])
        r()
        self.assertEqual(self.emitter.observers, [])


    def test_twoObservers(self):
        """
        Emitting an event when there are two observers causes two callbacks to
        occur.
        """
        self.observed = 0
        def _obs(event):
            self.observed += 1

        self.emitter.addObserver(_obs)
        self.emitter.addObserver(_obs)
        self.emitter.emitEvent(u'anEvent')
        self.assertEqual(self.observed, 2)
        self.assertEqual(len(self.emitter.observers), 2)


    def test_reentrancy(self):
        """
        Adding or removing observers from within an observer callback have no
        effect on the event currently being emitted.
        """
        def _obs(event):
            self.observed += 1

        def _obsWithRemove(event):
            self.observed += 1
            r()
            r2()

        def _obsWithAdd(event):
            self.observed += 1
            self.emitter.addObserver(_obsWithAdd)

        r = self.emitter.addObserver(_obsWithRemove)
        r2 = self.emitter.addObserver(_obs)

        self.observed = 0
        self.emitter.emitEvent(u'anEvent')
        self.assertEqual(self.observed, 2)
        self.assertEqual(len(self.emitter.observers), 0)

        self.emitter.addObserver(_obsWithAdd)
        self.observed = 0
        self.emitter.emitEvent(u'anEvent')
        self.assertEqual(self.observed, 1)
        self.assertEqual(len(self.emitter.observers), 2)

        self.observed = 0
        self.emitter.emitEvent(u'anEvent')
        self.assertEqual(self.observed, 2)
        self.assertEqual(len(self.emitter.observers), 4)



class DecimalFormatterTests(TestCase):
    """
    Tests for L{methanal.util.DecimalFormatter}.
    """
    def test_formatDefaults(self):
        """
        Formatting a value with the default settings produces a decimal number
        grouped 3 digits at a time.
        """
        f = DecimalFormatter()

        self.assertEquals(
            f.format(u''),
            u'')
        self.assertEquals(
            f.format(u'1'),
            u'1')
        self.assertEquals(
            f.format(u'123'),
            u'123')
        self.assertEquals(
            f.format(u'1234'),
            u'1,234')
        self.assertEquals(
            f.format(u'1234.56'),
            u'1,234.56')
        self.assertEquals(
            f.format(u'123456'),
            u'123,456')
        self.assertEquals(
            f.format(u'12345678'),
            u'12,345,678')
        self.assertEquals(
            f.format(u'1234567.89'),
            u'1,234,567.89')


    def test_formatCustom(self):
        """
        Formatting a value obeys the decimal grouping and separator
        specifications.
        """
        f = DecimalFormatter(grouping=[3, 2, -1])

        self.assertEquals(
            f.format(u''),
            u'')
        self.assertEquals(
            f.format(u'1'),
            u'1')
        self.assertEquals(
            f.format(u'123'),
            u'123')
        self.assertEquals(
            f.format(u'1234'),
            u'1,234')
        self.assertEquals(
            f.format(u'123456'),
            u'1,23,456')
        self.assertEquals(
            f.format(u'123456789'),
            u'1234,56,789')
        self.assertEquals(
            f.format(u'1234567.89'),
            u'12,34,567.89')

        f = DecimalFormatter(
            grouping=[3, 2, -1], thousandsSeparator=u'.',
            decimalSeparator=u',')
        self.assertEquals(
            f.format(u'123456'),
            u'1.23.456')
        self.assertEquals(
            f.format(u'123456789'),
            u'1234.56.789')
        self.assertEquals(
            f.format(u'1234567,89'),
            u'12.34.567,89')


    def test_formatNonString(self):
        """
        L{methanal.util.DecimalFormatter.format} can handle some non-string
        values, such as C{None}, C{int} and C{Decimal}.
        """
        f = DecimalFormatter(grouping=[3, 2, -1])
        self.assertEquals(
            f.format(None),
            u'')
        self.assertEquals(
            f.format(1),
            u'1')
        self.assertEquals(
            f.format(Decimal(123)),
            u'123')
        self.assertEquals(
            f.format(Decimal('1234')),
            u'1,234')

        f = DecimalFormatter(
            grouping=[3, 2, -1], thousandsSeparator=u'.',
            decimalSeparator=u',')
        self.assertEquals(
            f.format(123456),
            u'1.23.456')
        self.assertEquals(
            f.format(123456789),
            u'1234.56.789')



class CurrencyFormatterTests(TestCase):
    """
    Tests for L{methanal.util.CurrencyFormatter}.
    """
    def test_formatDefaults(self):
        """
        Formatting a value with the default settings produces the currency
        symbol followed by one space and the decimal number grouped 3 digits at
        a time.
        """
        f = CurrencyFormatter(u'X')

        self.assertEquals(
            f.format(u'1'),
            u'X 1')
        self.assertEquals(
            f.format(u'1234'),
            u'X 1,234')
        self.assertEquals(
            f.format(u'1234.56'),
            u'X 1,234.56')
        self.assertEquals(
            f.format(u'1234567'),
            u'X 1,234,567')


    def test_formatCustom(self):
        """
        Formatting a value obeys the decimal grouping, separator and currency
        symbol separator specifications.
        """
        f = CurrencyFormatter(
            symbol=u'X', symbolSeparator=u'', grouping=[3, 2, -1])

        self.assertEquals(
            f.format(u'1'),
            u'X1')
        self.assertEquals(
            f.format(u'1234'),
            u'X1,234')
        self.assertEquals(
            f.format(u'1234.56'),
            u'X1,234.56')
        self.assertEquals(
            f.format(u'1234567'),
            u'X12,34,567')
        self.assertEquals(
            f.format(u'1234567890'),
            u'X12345,67,890')



from twisted.python.filepath import FilePath

from xmantissa.offering import Offering

import methanal
from methanal.theme import Theme
from methanal.publicpage import MethanalPublicPage

plugin = Offering(
    name=u'Methanal',
    description=u'A forms library for Mantissa',
    siteRequirements=[],
    appPowerups=[MethanalPublicPage],
    installablePowerups=[],
    loginInterfaces=[],
    themes=[Theme('methanal-base', 0)],
    staticContentPath=FilePath(methanal.__file__).sibling('static'))



from twisted.trial import unittest

from methanal.imethanal import IEnumeration
from methanal import enums
# Adapter registration side-effect.
from methanal import view
# To quell Pyflakes' fears.
view



class EnumerationAdapterTests(unittest.TestCase):
    """
    Tests for L{IEnumeration} adapters.
    """
    def test_list(self):
        """
        Adapting a C{list} to L{IEnumeration} results in an L{Enum} accurately
        representing the list.
        """
        values = [
            (u'foo', u'Foo'),
            (u'bar', u'Bar')]
        enum = IEnumeration(values)
        self.assertEquals(enum.as_pairs(), values)
        for value, desc in values:
            item = enum.get(value)
            self.assertEquals(item.value, value)
            self.assertEquals(item.desc, desc)


    def test_groupList(self):
        """
        Adapting a C{list} of nested C{list}s, as used by
        L{methanal.view.GroupedSelectInput}, results in an L{Enum} with
        L{EnumItems} with a C{'group'} extra value the same as the first
        element in each C{tuple}. L{IEnumeration.as_pairs} returns a flat
        C{list} for nested C{list}s adapted to L{IEnumeration}.
        """
        values = [
            (u'Group', [
                (u'foo', u'Foo'),
                (u'bar', u'Bar')]),
            (u'Group 2', [
                (u'quux', u'Quux'),
                (u'frob', u'Frob')])]

        enum = IEnumeration(values)
        for groupName, innerValues in values:
            for value, desc in innerValues:
                item = enum.get(value)
                self.assertEquals(item.value, value)
                self.assertEquals(item.desc, desc)
                self.assertEquals(item.get('group'), groupName)

        pairs = sum(zip(*values)[1], [])
        self.assertEquals(enum.as_pairs(), pairs)


    def test_notAdapted(self):
        """
        Adapting C{tuple}, C{iter} or generator expression raises L{TypeError}.
        """
        values = (n for n in xrange(5))
        self.assertRaises(TypeError, IEnumeration, tuple(values))
        self.assertRaises(TypeError, IEnumeration, iter(values))
        self.assertRaises(TypeError, IEnumeration, values)



class DeprecationWarningsTests(unittest.TestCase):
    """
    Tests for L{methanal.enums} module deprecation warnings.
    """
    def test_enum(self):
        """
        L{methanal.enums.Enum} is deprecated.
        """
        enums.Enum
        warnings = self.flushWarnings([self.test_enum])
        self.assertEqual(len(warnings), 1)
        self.assertEqual(warnings[0]['category'], DeprecationWarning)
        self.assertIn(
            'methanal.enums.Enum was deprecated in Methanal 0.4.0',
            warnings[0]['message'])


    def test_objectEnum(self):
        """
        L{methanal.enums.ObjectEnum} is deprecated.
        """
        enums.ObjectEnum
        warnings = self.flushWarnings([self.test_objectEnum])
        self.assertEqual(len(warnings), 1)
        self.assertEqual(warnings[0]['category'], DeprecationWarning)
        self.assertIn(
            'methanal.enums.ObjectEnum was deprecated in Methanal 0.4.0',
            warnings[0]['message'])


    def test_enumItem(self):
        """
        L{methanal.enums.EnumItem} is deprecated.
        """
        enums.EnumItem
        warnings = self.flushWarnings([self.test_enumItem])
        self.assertEqual(len(warnings), 1)
        self.assertEqual(warnings[0]['category'], DeprecationWarning)
        self.assertIn(
            'methanal.enums.EnumItem was deprecated in Methanal 0.4.0',
            warnings[0]['message'])


    def test_invalidEnumItem(self):
        """
        L{methanal.enums.InvalidEnumItem} is deprecated.
        """
        enums.InvalidEnumItem
        warnings = self.flushWarnings([self.test_invalidEnumItem])
        self.assertEqual(len(warnings), 1)
        self.assertEqual(warnings[0]['category'], DeprecationWarning)
        self.assertIn(
            'methanal.enums.InvalidEnumItem was deprecated in Methanal 0.4.0',
            warnings[0]['message'])



from nevow.testutil import JavaScriptTestCase



class JSUnitTests(JavaScriptTestCase):
    def test_methanal(self):
        return u'Methanal.Tests.TestMethanal'


    def test_util(self):
        return u'Methanal.Tests.TestUtil'


    def test_validators(self):
        return u'Methanal.Tests.TestValidators'


    def test_preds(self):
        return u'Methanal.Tests.TestPreds'


    def test_view(self):
        return u'Methanal.Tests.TestView'


    def test_widgets(self):
        return u'Methanal.Tests.TestWidgets'



"""
Utility widgets designed to operate outside of forms.
"""
import time
from warnings import warn

from axiom.item import SQLAttribute
from epsilon.extime import FixedOffset, Time
from epsilon.structlike import record
from methanal.errors import InvalidIdentifier
from methanal.imethanal import IColumn
from methanal.model import Value
from methanal.util import getArgsDict
from methanal.view import (ActionButton, FormInput, LiveForm, SelectInput,
                           SimpleForm, liveFormFromAttributes)
from nevow.athena import expose
from nevow.inevow import IAthenaTransportable
from nevow.page import renderer
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.python.components import registerAdapter
from twisted.python.deprecate import deprecated
from twisted.python.versions import Version
from xmantissa.ixmantissa import IColumn as mantissaIColumn
from xmantissa.ixmantissa import IWebTranslator
from xmantissa.webtheme import ThemedElement
from zope.interface import implements


class TimeTransportable(object):
    """
    An C{IAthenaTransportable} implementation for L{Time} instances.
    """
    implements(IAthenaTransportable)

    jsClass = u'Methanal.Util.Time.fromTimestamp'


    def __init__(self, time):
        self.time = time


    def getInitialArguments(self):
        return [self.time.asPOSIXTimestamp() * 1000]

registerAdapter(TimeTransportable, Time, IAthenaTransportable)



class AttributeColumn(object):
    """
    An L{methanal.imethanal.IColumn} provider for Axiom attributes.

    @type attribute: L{axiom.attributes.SQLAttribute}

    @type attributeID: C{str}
    @param attributeID: Attribute column identifier, defaults to the attribute
        name
    """
    implements(IColumn)

    def __init__(self, attribute, attributeID=None, title=None):
        self.attribute = attribute
        if attributeID is None:
            attributeID = attribute.attrname
        self.attributeID = attributeID
        if title is None:
            title = getattr(self.attribute, 'doc', None)
            if not title:
                title = unicode(attributeID, 'ascii')
        self.title = title


    # IColumn

    def extractValue(self, model, item):
        """
        Extract a simple value for this column from a given item, suitable for
        serialization via Athena's client-communication layer.

        This implementation differs from the one in Mantissa in that it uses
        C{getattr}, instead of C{__get__}, thus allowing it to work on items
        wrapped in a C{SharedProxy}.

        @param model: The query list object requesting the value

        @param item: An instance of the class that this L{AttributeColumn}'s
            L{attribute} was taken from, to retrieve the value from

        @return: A value of an attribute of C{item}, of a type dependent upon
            this L{AttributeColumn}'s L{attribute}
        """
        return getattr(item, self.attribute.attrname)


    def extractLink(self, model, item):
        webTranslator = IWebTranslator(item.store, None)
        if webTranslator is not None:
            return unicode(webTranslator.toWebID(item), 'ascii')
        return None


    def getType(self):
        return type(self.attribute).__name__

registerAdapter(AttributeColumn, SQLAttribute, IColumn)



class LinkColumn(object):
    """
    Provide a custom link for an existing L{IColumn}.

    @type column: L{IColumn}
    @ivar column: Existing column to provide a custom link for

    @ivar extractLink: A callable matching the signature of
        L{IColumn.extractLink}
    """
    implements(IColumn)

    def __init__(self, column, extractLink):
        self._column = column
        self.extractLink = extractLink
        self.extractValue = self._column.extractValue
        self.getType = self._column.getType
        self.attributeID = self._column.attributeID
        self.title = self._column.title



class Row(object):
    """
    A L{Table} row.

    @ivar id: Row identifier

    @type cells: Mapping of C{unicode} to L{Cell}
    @ivar cells: Mapping of column identifiers to cell objects
    """
    def __init__(self, item, index, table):
        self.id = index
        self.cells = dict()
        for column in table.columns:
            columnID = unicode(column.attributeID, 'ascii')
            value = column.extractValue(table, item)
            link = column.extractLink(table, item)
            self.cells[columnID] = Cell(value, link)



class RowTransportable(record('row')):
    """
    An C{IAthenaTransportable} implementation for L{Row} instances.
    """
    implements(IAthenaTransportable)

    jsClass = u'Methanal.Widgets.Row'

    def getInitialArguments(self):
        return [self.row.id, self.row.cells]

registerAdapter(RowTransportable, Row, IAthenaTransportable)



class Cell(object):
    """
    A L{Table} cell.

    @ivar value: Cell value

    @type link: C{unicode}
    @ivar link: Hyperlink for the cell, or C{None} if the cell is not
        hyperlinked
    """
    def __init__(self, value, link):
        self.value = value
        self.link = link



class CellTransportable(record('cell')):
    """
    An C{IAthenaTransportable} implementation for L{Cell} instances.
    """
    implements(IAthenaTransportable)

    jsClass = u'Methanal.Widgets.Cell'

    def getInitialArguments(self):
        return [self.cell.value, self.cell.link]

registerAdapter(CellTransportable, Cell, IAthenaTransportable)



class ColumnTransportable(record('column')):
    """
    An C{IAthenaTransportable} implementation for L{IColumn}.
    """
    implements(IAthenaTransportable)

    columnTypes = {
        'text': u'Methanal.Widgets.TextColumn',
        'integer': u'Methanal.Widgets.IntegerColumn',
        'ieee754_double': u'Methanal.Widgets.FloatColumn',
        'boolean': u'Methanal.Widgets.BooleanColumn',
        'timestamp': u'Methanal.Widgets.TimestampColumn'}

    @property
    def jsClass(self):
        """
        Determine the Javascript class name based on the column type.
        """
        columnType = self.column.getType()
        if columnType is None:
            columnType = 'text'
        return self.columnTypes.get(columnType)


    def getInitialArguments(self):
        columnID = unicode(self.column.attributeID, 'ascii')
        columnType = self.column.getType()
        if columnType is not None:
            columnType = unicode(columnType, 'ascii')
        return [columnID, self.column.title, columnType]

registerAdapter(ColumnTransportable, IColumn, IAthenaTransportable)



class Table(ThemedElement):
    """
    Tabulate data with column values derived from Items.

    @type _itemFactory: I{callable}, taking no arguments, that produces an
        I{iterable} of L{axiom.item.Item}.

    @type columns: I{iterable} of L{methanal.imethanal.IColumn}
    """
    jsClass = u'Methanal.Widgets.Table'
    fragmentName = 'methanal-table'


    def __init__(self, items, columns, **kw):
        super(Table, self).__init__(**kw)
        if callable(items):
            itemsFactory = items
        else:
            items = list(items)
            itemsFactory = lambda: items
        self._itemsFactory = itemsFactory
        self.columns = [IColumn(column) for column in columns]


    @property
    def items(self):
        """
        An  I{iterable} of L{axiom.item.Item}.
        """
        return self._itemsFactory()


    def getInitialArguments(self):
        return [getArgsDict(self)]


    def getArgs(self):
        return {u'columns': self.columns,
                u'rows': self._createRows(self.items)}


    def _createRows(self, items):
        """
        Create L{Row} items for use on the client.
        """
        return [
            Row(item, index, self) for index, item in enumerate(items)]


    def replaceRemoteRows(self, items=None):
        """
        Replace rows on the remote side.

        @type  items: I{iterable} of L{axiom.item.Item}
        @param items: Rows.
        """
        if items is None:
            items = self.items
        return self.callRemote('repopulate', self._createRows(items))


    @expose
    def performAction(self, name, rowIndex):
        method = getattr(self, 'action_' + name)
        item = self.items[rowIndex]
        return method(item)


    @expose
    def _getRows(self):
        return self._createRows(self.items)



class QueryList(ThemedElement):
    """
    A widget that displays data tabulated according to a set of columns.

    Actions are supported too.
    """
    jsClass = u'Methanal.Widgets.QueryList'
    fragmentName = 'methanal-table'

    def __init__(self, rows, columns, webTranslator=None, timezone=None, **kw):
        warn('QueryList is deprecated, use methanal.widgets.Table instead')
        super(QueryList, self).__init__(**kw)

        self.rows = list(rows)
        def _adapt(col):
            try:
                return IColumn(col)
            except TypeError:
                col = mantissaIColumn(col)

            warn('use methanal.imethanal.IColumn instead of '
                 'xmantissa.ixmantissa.IColumn', DeprecationWarning, 2)
            return col

        columns = (_adapt(col) for col in columns)
        self.columns = [(col.attributeID.decode('ascii'), col)
                        for col in columns]
        self.webTranslator = webTranslator

        if timezone is None:
            hour, minute = divmod(time.timezone, -3600)
            timezone = FixedOffset(hour, minute)
            warn('not passing in timezone is deprecated', DeprecationWarning, 2)

        self.timezone = timezone


    def dictifyItem(self, item, index):
        def _formatValue(value):
            if isinstance(value, Time):
                return value.asDatetime(self.timezone).strftime(
                    '%a, %d %h %Y %H:%M:%S').decode('ascii')
            return value

        if isinstance(item, tuple):
            link, item = item
        else:
            if self.webTranslator is None:
                self.webTranslator = IWebTranslator(item.store)
            link = unicode(self.webTranslator.toWebID(item), 'ascii')

        d = dict((cid, _formatValue(col.extractValue(self, item)))
                 for (cid, col) in self.columns)
        d[u'__link__'] = link
        d[u'__id__'] = index

        return d


    def getInitialArguments(self):
        return [getArgsDict(self)]


    def getArgs(self):
        IDs = []
        aliases = {}
        for cid, col in self.columns:
            IDs.append(cid)
            if isinstance(col, AttributeColumn):
                alias = col.attribute.doc
            else:
                alias = col
            aliases[cid] = unicode(alias)

        return {u'columnIDs':     IDs,
                u'columnAliases': aliases,
                u'rows':          [self.dictifyItem(row, i)
                                   for i, row in enumerate(self.rows)]}


    @expose
    def performAction(self, name, rowIndex):
        method = getattr(self, 'action_' + name)
        item = self.rows[rowIndex]
        return method(item)



class FilterList(ThemedElement):
    """
    A filtering search widget.

    Essentially just a form that results in a server-side call, on submission,
    and a result widget.

    One particularly common application is a search widget: A form containing
    inputs representing fields to filter by, which, when submitted, results in
    a server-side database query and a QueryList widget.
    """
    jsClass = u'Methanal.Widgets.FilterList'
    fragmentName = 'methanal-filter-list'

    def __init__(self, form, resultWidget, title=None, **kw):
        """
        Initialise the filter widget.

        @type form: C{methanal.view.LiveForm}
        @param form: Form to display for filter inputs, the form's
            C{submitSuccess} client method will be passed the result
            widget; considering deriving client objects from
            C{Methanal.Widgets.FilterListForm}

        @type resultWidget: C{callable}
        @param resultWidget: A callable passed the result of C{form}'s
            callback and expected to return a renderable representing the
            filter results

        @type title: C{unicode}
        @param title: A title to display along with the filter widget
        """
        super(FilterList, self).__init__(**kw)
        self.form = form
        self.resultWidget = resultWidget
        if title is None:
            title = self.form.doc
        self.title = title

        self.originalCallback = self.form.model.callback
        self.form.model.callback = self.filterCallback


    def filterCallback(self, **kw):
        """
        Handle form submission.

        Call the original form callback and create the result widget.

        @rtype: C{Deferred}
        """
        def makeResultWidget(data):
            w = self.resultWidget(data)
            w.setFragmentParent(self)
            return w

        return maybeDeferred(self.originalCallback, **kw
            ).addCallback(makeResultWidget)


    @renderer
    def filterForm(self, req, tag):
        self.form.setFragmentParent(self)
        return self.form



class SimpleFilterList(FilterList):
    """
    A simple L{FilterList} implementation.

    Intended to generate a C{LiveForm} from a sequence of attributes, call a
    user-specified callback and display the results in a L{QueryList} with the
    desired columns.
    """
    def __init__(self, store, filterAttrs, callback, resultColumns,
                 timezone=None, webTranslator=None, **kw):
        """
        Initialise the filter widget.

        @type store: C{axiom.store.Store}
        @param store: Store that the specified attributes reside in

        @type filterAttrs: C{sequence} of Axiom attributes
        @param filterAttrs: The attributes to provide form inputs for the
            attributes to filter on

        @type callback: C{callable} returning an C{iterable} of
            C{axiom.item.Item}s
        @param callback: The callable that is triggered when the filter form
            is submitted, passed parameters named according to the
            attribute names specified by the attributes in L{filterAttrs},
            returning result items

        @type resultColumns: C{list} of L{methanal.imethanal.IColumn}
        @param resultColumns: Columns for display in the result widget

        @type timezone: C{tzinfo}
        @param timezone: Timezone used for displaying timestamps.

        @type webTranslator: L{xmantissa.ixmantissa.IWebTranslator}
        @param webTranslator: The translator used for linking items.
        """
        form = liveFormFromAttributes(store=store,
                                      attributes=filterAttrs,
                                      callback=callback,
                                      doc=u'Filter',
                                      timezone=timezone)
        form.jsClass = u'Methanal.Widgets.FilterListForm'

        resultWidget = lambda rows: QueryList(rows=rows,
                                              columns=resultColumns,
                                              webTranslator=webTranslator,
                                              timezone=timezone)

        super(SimpleFilterList, self).__init__(form=form,
                                               resultWidget=resultWidget,
                                               **kw)



class Lookup(FormInput):
    """
    Complex input for arriving at a single result, given a set of criteria.

    @type form: L{LookupForm}
    @ivar form: Form containing the inputs for capturing lookup data, the
        result input will be automatically added to the form.

    @type populate: C{callable} taking C{dict} returning an C{iterable} of
        L{LookupResultTransportable}
    @ivar populate: Callback called with a mapping of form input names to
        criteria, returning L{LookupResultTransportable} instances.
    """
    fragmentName = 'methanal-lookup'
    jsClass = u'Methanal.Widgets.Lookup'


    def __init__(self, form, populate, values=None, **kw):
        """
        Lookup input.

        @param values: Initial values for results input, see
            L{SelectInput.values} for more information.
        """
        super(Lookup, self).__init__(**kw)
        self.form = form
        self.populate = populate
        if values is None:
            values = []

        self.alterModel()
        self.alterForm(values)


    def alterModel(self):
        """
        Alter the lookup form model.
        """
        self.form.model.params['__result__'] = Value(
            name=u'__result__', doc=u'Result')


    def alterForm(self, values):
        """
        Alter the lookup form.
        """
        SelectInput(parent=self.form, name='__result__', values=values)


    @expose
    def lookup(self, data):
        """
        Call L{populate} with the lookup criteria and return the results.
        """
        return list(self.populate(data))


    @renderer
    def criteriaForm(self, req, tag):
        self.form.setFragmentParent(self)
        return tag[self.form]



class LookupResultTransportable(object):
    """
    An C{IAthenaTransportable} implementation base class for L{Lookup} results.

    @type id: C{unicode}
    @ivar id: Unique result identifier.

    @type values: C{dict} mapping C{unicode} to values.
    @ivar values: Result values, these will be reverse populated to the
        L{LookupForm} when initialising the control.

    @type name: C{unicode}
    @ivar name: Human readable result name, or C{None} to use a represenation
        of L{values}, Defaults to C{None}.
    """
    implements(IAthenaTransportable)

    jsClass = u'Methanal.Widgets.LookupResult'


    def __init__(self, id, values, name=None):
        self.id = id
        self.values = values
        self.name = name


    def getInitialArguments(self):
        return [self.id, self.values, self.name]



class LookupForm(SimpleForm):
    """
    L{SimpleForm} intended to be used as the C{form} parameter to L{Lookup}.
    """
    jsClass = u'Methanal.Widgets.LookupForm'



class ModalDialog(ThemedElement):
    """
    Modal dialog widget.

    @type title: C{unicode}
    @ivar title: Dialog title

    @type content: C{nevow.athena.LiveElement}
    @ivar content: Athena widget to serve as the content for the dialog
    """
    jsClass = u'Methanal.Widgets.ModalDialog'
    fragmentName = 'methanal-modal-dialog'

    def __init__(self, title, content, **kw):
        super(ModalDialog, self).__init__(**kw)
        self.title = title
        self.content = content
        self._notifyDismissed = []


    def connectionLost(self, reason):
        listeners = list(self._notifyDismissed)
        self._notifyDismissed = []
        for d in listeners:
            d.callback(reason)


    def whenDismissed(self):
        """
        Return a `Deferred` that fires when the dialog has been dismissed in
        any fashion on the client side.

        It is worth noting that this deferred will not be triggered if the
        widget's ``connectionMade`` method is never invoked, for example when
        statically rendered.

        :return: Deferred that fires with the reason for dismissal when the
        dialog is dismissed.
        """
        d = Deferred()
        self._notifyDismissed.append(d)
        return d


    @renderer
    def dialogTitle(self, req, tag):
        return tag[self.title]


    @renderer
    def dialogContent(self, req, tag):
        self.content.setFragmentParent(self)
        return tag[self.content]



class CancelAction(ActionButton):
    """
    Form action for dismissing a dialog.
    """
    jsClass = u'Methanal.Widgets.CancelAction'
    defaultName = u'Cancel'
    allowViewOnly = True



class ModalDialogForm(LiveForm):
    """
    L{methanal.view.LiveForm} for L{methanal.widgets.ModalDialog}.

    @type addCancelAction: C{bool}
    @ivar addCancelAction: Add the cancel action? Defaults to C{True}.
    """
    jsClass = u'Methanal.Widgets.ModalDialogForm'

    def __init__(self, addCancelAction=True, **kw):
        super(ModalDialogForm, self).__init__(**kw)
        if addCancelAction:
            self.actions.addAction(CancelAction())



class TabView(ThemedElement):
    """
    A tab container, visually displayed as a horizontal tab bar.

    Only one sub-container can be visible at a time.

    @type tabs: C{list} of L{methanal.widgets.Tab}
    @ivar tabs: Tabs to manage.

    @type topLevel: C{bool}
    @ivar topLevel: Is this a top-level TabView? Top-level TabViews will use
        the fragment part of the current URL to track which tab is selected,
        this behaviour supercedes L{Tab.selected}.

    @type _tabsByID: C{dict} mapping C{unicode} to L{methanal.widgets.Tab}
    @ivar _tabsByID: Mapping of unique tab IDs to tabs currently being managed.

    @type _tabGroups: C{dict} mapping C{unicode} to
        L{methanal.widgets.TabGroup}
    @ivar _tabGroups: Mapping of tab group identifiers to tab groups.
    """
    fragmentName = 'methanal-tab-view'
    jsClass = u'Methanal.Widgets.TabView'

    def __init__(self, tabs, topLevel=False, **kw):
        """
        @type  tabs: C{list} of L{methanal.widgets.Tab} or
            L{methanal.widgets.TabGroup}.
        @param tabs: Tab or tab groups to manage.
        """
        super(TabView, self).__init__(**kw)
        self._tabsByID = {}
        self._tabGroups = {}
        self.tabs = []
        self.topLevel = topLevel

        for tabOrGroup in tabs:
            if isinstance(tabOrGroup, TabGroup):
                self._manageGroup(tabOrGroup)
                for tab in tabOrGroup.tabs:
                    self._manageTab(tab)
            else:
                self._manageTab(tabOrGroup)


    def __repr__(self):
        return '<%s topLevel=%r tabs=%r>' % (
            type(self).__name__,
            self.topLevel,
            self.tabs)


    def _manageGroup(self, group):
        """
        Begin managing a L{methanal.widgets.TabGroup}.

        Tabs contained in a group are B{not} managed automatically,
        L{_manageTab} should be called for each tab.
        """
        self._tabGroups[group.id] = group


    def _manageTab(self, tab, overwrite=False):
        """
        Begin managing a L{methanal.widgets.Tab} widget.

        If a tab identifier is already being managed it is released before
        managing the new widget.
        """
        if tab.id in self._tabsByID:
            self._releaseTab(self.getTab(tab.id))
        self._tabsByID[tab.id] = tab
        self.tabs.append(tab)
        group = self._tabGroups.get(tab.group)
        if group is not None:
            group._manageTab(tab)


    def _releaseTab(self, tab):
        """
        Stop managing a L{methanal.widgets.Tab} widget.
        """
        if tab not in self.tabs:
            raise ValueError(
                '%r is not managed by %r' % (tab.id, self))
        del self._tabsByID[tab.id]
        self.tabs.remove(tab)
        group = self._tabGroups.get(tab.group)
        if group is not None:
            group._releaseTab(tab)


    def getTab(self, id):
        """
        Get a L{methanal.widgets.Tab} by its unique identifier.
        """
        tab = self._tabsByID.get(id)
        if tab is None:
            raise InvalidIdentifier(
                u'%r is not a valid tab identifier in %r' % (id, self))
        return tab


    def getTabIDs(self):
        """
        Get a C{list} of all the tab IDs in the group.
        """
        return [tab.id for tab in self.tabs]


    def getGroup(self, id):
        """
        Get a L{methanal.widgets.TabGroup} by its unique identifier.
        """
        group = self._tabGroups.get(id)
        if group is None:
            raise InvalidIdentifier(
                u'%r is not a valid group identifier in %r' % (id, self))
        return group


    @deprecated(Version('methanal', 0, 2, 1))
    def appendTab(self, tab):
        """
        Append a L{methanal.widgets.Tab} widget.

        @return: A C{Deferred} that fires when the widget has been inserted on
            the client side.
        """
        return self.updateTabs([tab])


    def updateTabs(self, tabs, tabsToRemove=None):
        """
        Update many L{methanal.widgets.Tab} widgets.

        All tab widgets are passed to the client side, updated there (or
        appended if they didn't previously exist) and added to the relevant
        groups. Use L{methanal.widgets.TabGroup.mergeGroups} to simulate adding
        to an existing group.

        @return: A C{Deferred} that fires when the widgets have been updated on
            the client side.
        """
        for tab in tabs:
            self._manageTab(tab)
            tab.setFragmentParent(self)

        tabIDsToRemove = []
        if tabsToRemove is not None:
            tabIDsToRemove = [tab.id for tab in tabsToRemove]

        return self.callRemote(
            '_updateTabsFromServer', tabs, tabIDsToRemove, self._tabGroups)


    appendTabs = deprecated(Version('methanal', 0, 2, 1))(updateTabs)


    def removeTabs(self, tabs):
        """
        Remove many L{methanal.widgets.Tab} widgets.

        Empty groups, caused by removing all contained tabs, are removed too.

        @return: A C{Deferred} that fires when the widget has been removed on
            the client side.
        """
        tabIDs = []
        for tab in tabs:
            tabIDs.append(tab.id)
            self._releaseTab(tab)

        return self.callRemote('_removeTabsFromServer', tabIDs, self._tabGroups)


    def updateGroup(self, group):
        """
        Update a L{methanal.widgets.TabGroup} and its tabs.

        Tabs specified in C{group} will replace those previously specified by
        another group of the same name.

        @return: A C{Deferred} that fires when the widgets have been inserted on
            the client side.
        """
        try:
            oldGroup = self.getGroup(group.id)
        except InvalidIdentifier:
            oldGroup = None

        tabsToRemove = []
        if oldGroup is not None:
            tabsToRemove = map(
                self.getTab,
                set(oldGroup.getTabIDs()) - set(group.getTabIDs()))
            for tab in tabsToRemove:
                self._releaseTab(tab)

        self._manageGroup(group)
        return self.updateTabs(group.tabs, tabsToRemove)


    appendGroup = deprecated(Version('methanal', 0, 2, 1))(updateGroup)


    def getInitialArguments(self):
        return [
            dict.fromkeys(self._tabsByID.iterkeys(), True),
            self._tabGroups,
            self.topLevel]


    @renderer
    def tabContents(self, req, tag):
        def _tabs():
            for tab in self.tabs:
                tab.setFragmentParent(self)
                yield tab

        return tag[_tabs()]



class TabGroup(object):
    """
    Visually group labels of L{methanal.widgets.Tab}s together.

    @type id: C{unicode}
    @ivar id: Unique identifier.

    @type title: C{unicode}
    @ivar title: Title of the group, used by L{methanal.widgets.TabView} when
        constructing the tab list.

    @type tabs: C{list} of L{methanal.widgets.Tab}
    @ivar tabs: Tabs to group together.
    """
    implements(IAthenaTransportable)

    jsClass = u'Methanal.Widgets.TabGroup'

    def __init__(self, id, title, tabs):
        self.id = id
        self.title = title
        self.tabs = []
        for tab in tabs:
            self._manageTab(tab)


    def __repr__(self):
        return '<%s id=%r title=%r tabs=%r>' % (
            type(self).__name__,
            self.id,
            self.title,
            self.tabs)


    def _manageTab(self, tab):
        """
        Manage a L{methanal.widgets.Tab} widget.
        """
        if tab not in self.tabs:
            self.tabs.append(tab)
            tab.group = self.id


    def _releaseTab(self, tab):
        """
        Stop managing a L{methanal.widgets.Tab} widget.
        """
        if tab in self.tabs:
            self.tabs.remove(tab)
            tab.group = None


    def getTabIDs(self):
        """
        Get a C{list} of all the tab IDs in the group.
        """
        return [tab.id for tab in self.tabs]


    @classmethod
    def mergeGroups(cls, old, new):
        """
        Merge two L{methanal.widgets.TabGroup}s together.
        """
        return cls(new.id, new.title, old.tabs + new.tabs)


    # IAthenaTransportable

    def getInitialArguments(self):
        return [self.id, self.title, self.getTabIDs()]



class Tab(ThemedElement):
    """
    A content container, intended to be passed to L{TabView}.

    @type id: C{unicode}
    @ivar id: Unique identifier for this container.

    @type title: C{unicode}
    @ivar title: Title of the container, used by L{TabView} when constructing
        the tab list.

    @ivar contentFactory: C{callable} taking no arguments that returns a widget
        to use for the tab content

    @type selected: C{bool}
    @ivar selected: Is this container to be selected initially? Defaults to
        C{False}.

    @type group: C{unicode}
    @ivar group: Identifier of the group this tab belongs to, or C{None} for no
        grouping. Defaults to C{None}.
    """
    fragmentName = 'methanal-tab'

    def __init__(self, id, title, contentFactory, selected=False, group=None,
                 **kw):
        super(Tab, self).__init__(**kw)

        self.id = id
        self.title = title
        self.contentFactory = contentFactory
        self.selected = selected
        self.group = group


    def __repr__(self):
        return '<%s id=%r title=%r selected=%r group=%r>' % (
            type(self).__name__,
            self.id,
            self.title,
            self.selected,
            self.group)


    def getInitialArguments(self):
        return [getArgsDict(self)]


    def getArgs(self):
        return {u'id': self.id,
                u'title': self.title,
                u'selected': self.selected,
                u'group': self.group}


    @expose
    def getContent(self, nodeID=None):
        def _setFragmentParent(content):
            content.setFragmentParent(self)
            return content
        d = maybeDeferred(self.contentFactory)
        d.addCallback(_setFragmentParent)
        return d


    @renderer
    def tabContent(self, req, tag):
        return tag



class StaticTab(Tab):
    """
    Static content tab container.

    Content is inserted at render time and doesn't change or reload.
    """
    jsClass = u'Methanal.Widgets.StaticTab'

    def __init__(self, content=None, **kw):
        """
        @type  content: C{nevow.athena.LiveElement}
        @param content: Optional static content to use. Either C{content} or
            L{contentFactory} must be specified.
        """
        if content is not None:
            kw['contentFactory'] = lambda: content
        super(StaticTab, self).__init__(**kw)


    @renderer
    def tabContent(self, req, tag):
        d = self.getContent()
        d.addCallback(tag.__getitem__)
        return d


    def updateRemoteContent(self):
        pass



class DynamicTab(Tab):
    """
    Dynamic content tab container.

    Content is only requested, from the server, and inserted once the tab
    widget has been inserted into the document on the client side.
    """
    jsClass = u'Methanal.Widgets.DynamicTab'



class DemandTab(Tab):
    """
    On-demand content tab container.

    Content is only requested, from the server, and inserted when the tab is
    selected. Selecting the tab always retrieves new content; selecting the tab
    before a previous fetch attempt has completed will result in that data
    being discarded and a new fetch occurring.
    """
    jsClass = u'Methanal.Widgets.DemandTab'

    def updateRemoteContent(self):
        """
        Force the remote content to be updated.
        """
        def _callRemote(content):
            return self.callRemote(
                'setContentFromWidgetInfo', content, u'content')
        d = self.getContent()
        d.addCallback(_callRemote)
        return d



class Expander(ThemedElement):
    """
    Collapsable container with static content.

    The container's header and content can be updated by calling
    L{updateRemoteHeaderContent} and L{updateRemoteContent} respectively.

    @type headerFactory: C{callable} returning a C{nevow.athena.LiveElement}
    @ivar headerFactory: Factory for the container header;
        L{methanal.widgets.ExpanderHeader} is a simple header widget.

    @type contentFactory: C{callable} returning a C{nevow.athena.LiveElement}
    @ivar contentFactory: Factory for the container content.

    @type expanded: C{bool}
    @ivar expanded: Is the content visible?

    @ivar currentHeaderContent: Current header renderable.

    @ivar currentExpanderContent: Current header renderable.
    """
    fragmentName = 'methanal-expander'
    jsClass = u'Methanal.Widgets.Expander'


    def __init__(self, headerFactory, contentFactory, expanded=False, **kw):
        super(Expander, self).__init__(**kw)
        self.headerFactory = headerFactory
        self.contentFactory = contentFactory
        self.expanded = expanded
        self.currentHeaderContent = None
        self.currentExpanderContent = None


    def getInitialArguments(self):
        return [self.expanded]


    @expose
    def getContent(self, nodeID):
        """
        Get the content for C{nodeID}.

        @rtype: I{renderable}
        """
        return {
            'header':  self.getHeaderContent,
            'content': self.getExpanderContent}[nodeID]()


    def getHeaderContent(self):
        """
        Get the content for the expander header.

        The content factory is invoked and its result compared to
        L{currentHeaderContent}, with C{__eq__}, if the result is C{True}
        then C{None} is returned.

        @return: Renderable or C{None} if the content hasn't changed.
        """
        content = self.headerFactory()
        if content == self.currentHeaderContent:
            return None
        content.setFragmentParent(self)
        self.currentHeaderContent = content
        return content


    def getExpanderContent(self):
        """
        Get the content for the expander body.

        The content factory is invoked and its result compared to
        L{currentExpanderContent}, with C{__eq__}, if the result is C{True}
        then C{None} is returned.

        @return: Renderable or C{None} if the content hasn't changed.
        """
        content = self.contentFactory()
        if content == self.currentExpanderContent:
            return None
        content.setFragmentParent(self)
        self.currentExpanderContent = content
        return content


    def updateRemoteHeaderContent(self):
        """
        Update the client-side header content.

        If L{getHeaderContent} returns C{None} no update is performed.
        """
        content = self.getHeaderContent()
        if content is not None:
            return self.callRemote(
                'setContentFromWidgetInfo', content, u'header')


    def updateRemoteContent(self):
        """
        Update the client-side body content.

        If L{getExpanderContent} returns C{None} no update is performed.
        """
        content = self.getExpanderContent()
        if content is not None:
            return self.callRemote(
                'setContentFromWidgetInfo', content, u'content')


    @renderer
    def headerContent(self, req, tag):
        return tag[self.getHeaderContent()]


    @renderer
    def expanderContent(self, req, tag):
        return tag[self.getExpanderContent()]



class DynamicExpander(Expander):
    """
    Collapsable container with dynamically loaded content upon being expanded
    for the first time.
    """
    jsClass = u'Methanal.Widgets.DynamicExpander'


    @renderer
    def expanderContent(self, req, tag):
        return tag



class DemandExpander(DynamicExpander):
    """
    Collapsable container with dynamically loaded content upon being expanded.
    """
    jsClass = u'Methanal.Widgets.DemandExpander'



class ExpanderHeader(ThemedElement):
    """
    Simple header for L{methanal.widgets.Expander}.

    @ivar label: Renderable header label.
    """
    fragmentName = 'methanal-expander-header'


    def __init__(self, label, **kw):
        super(ExpanderHeader, self).__init__(**kw)
        self.label = label


    @renderer
    def content(self, req, tag):
        tag.fillSlots('label', self.label)
        return tag



from zope.interface import implements

from twisted.python.filepath import FilePath

from axiom.item import Item
from axiom.attributes import integer

from nevow.tags import h1
from nevow.static import File

from xmantissa.ixmantissa import IPublicPage, ICustomizable, IStaticShellContent
from xmantissa.publicresource import PublicPage

import methanal



class MethanalPublicPage(Item):
    implements(IPublicPage)

    typeName = 'methanal_public_page'
    schemaVersion = 1
    powerupInterfaces = (IPublicPage,)

    dummy = integer()


    def getResource(self):
        return PublicIndexPage(self, IStaticShellContent(self.store, None))



class PublicIndexPage(PublicPage):
    implements(ICustomizable)
    title = 'Methanal'


    def __init__(self, original, staticContent, forUser=None):
        super(PublicIndexPage, self).__init__(
            original, original.store.parent, h1['Methanal'], staticContent,
            forUser)


    def child_static(self, ctx):
        s = FilePath(methanal.__file__).sibling('static')
        return File(s.path)


    def customizeFor(self, forUser):
        return type(self)(self.original, self.staticContent, forUser)



"""
Unit tests for L{methanal.widgets}.
"""
from functools import partial

from twisted.trial import unittest

from nevow import inevow

from xmantissa.webtheme import ThemedElement

from methanal import widgets, errors



class TabViewTests(unittest.TestCase):
    """
    Tests for L{methanal.widgets.TabView}.
    """
    def setUp(self):
        self.contentFactory = lambda: []
        self.tabs = [
            widgets.Tab(u'id1', u'Title 1', self.contentFactory),
            widgets.Tab(u'id2', u'Title 2', self.contentFactory),
            widgets.Tab(u'id3', u'Title 3', self.contentFactory)]
        self.tabView = widgets.TabView(self.tabs)


    def test_create(self):
        """
        Creating a L{TabView} widget initialises L{TabView._tabsByID} and
        L{TabView.tabs} with the values originally specified.
        """
        self.assertEquals(
            sorted(self.tabView._tabsByID.keys()),
            [u'id1', u'id2', u'id3'])
        self.assertEquals(
            self.tabView.tabs,
            self.tabs)


    def test_createGroup(self):
        """
        Creating a L{methanal.widgets.TabView} widget initialises
        L{methanal.widgets.TabView._tabsByID}, L{methanal.widgets.TabView.tabs}
        and L{methanal.widgets.TabView._tabGroups} with the values originally
        specified, tab groups are merged in and managed.
        """
        tabGroup = widgets.TabGroup(u'group1', u'Group', tabs=[
            widgets.Tab(u'id4', u'Title 4', self.contentFactory)])
        tabs = self.tabs + [
            tabGroup,
            widgets.Tab(u'id5', u'Title 5', self.contentFactory)]
        tabView = widgets.TabView(tabs)
        self.assertEquals(
            tabView.getTabIDs(),
            [u'id1', u'id2', u'id3', u'id4', u'id5'])
        self.assertEquals(
            tabView._tabGroups,
            {u'group1': tabGroup})


    def test_updateTabs(self):
        """
        Updating tabs on the server side manages them and invokes methods
        on the client side to insert them.
        """
        self.result = None

        def callRemote(methodName, *a):
            self.result = methodName == '_updateTabsFromServer'

        tab = widgets.Tab(u'id4', u'Title 4', self.contentFactory)
        self.patch(self.tabView, 'callRemote', callRemote)
        self.tabView.updateTabs([tab])
        self.assertTrue(self.result)
        self.assertNotIdentical(self.tabView.getTab(u'id4'), None)
        self.assertIdentical(self.tabView.tabs[-1], tab)

        replacementTab = widgets.Tab(u'id1', u'New title', self.contentFactory)
        oldTab = self.tabView.getTab(u'id1')
        self.tabView.updateTabs([replacementTab])
        self.assertIdentical(self.tabView.getTab(u'id1'), replacementTab)
        self.assertNotIn(oldTab, self.tabView.tabs)


    def test_updateGroup(self):
        """
        Updating a group on the server site manages it, and all the tabs it
        contains, and invokes methods on the client side to insert them.
        """
        self.result = None

        def callRemote(methodName, *a):
            self.result = methodName == '_updateTabsFromServer'

        tab = widgets.Tab(u'id4', u'Title 4', self.contentFactory)
        group = widgets.TabGroup(u'group1', u'Group', tabs=[tab])
        self.patch(self.tabView, 'callRemote', callRemote)
        self.tabView.updateGroup(group)
        self.assertTrue(self.result)
        self.assertNotIdentical(self.tabView.getTab(u'id4'), None)
        self.assertNotIdentical(self.tabView.getGroup(u'group1'), None)
        self.assertIdentical(self.tabView.tabs[-1], tab)

        # Update a group, and add a new tab.
        newTab = widgets.Tab(u'id5', u'Title 5', self.contentFactory)
        replacementGroup = widgets.TabGroup(
            u'group1', u'New Group', tabs=[newTab])
        self.tabView.updateGroup(replacementGroup)
        self.assertIdentical(
            self.tabView.getGroup(u'group1'), replacementGroup)
        self.assertNotIdentical(self.tabView.getTab(u'id5'), None)
        self.assertRaises(
            errors.InvalidIdentifier, self.tabView.getTab, u'id4')
        self.assertNotIn(tab, self.tabView.tabs)

        # Remove a tab from a group.
        self.tabView.removeTabs([newTab])
        self.assertRaises(
            errors.InvalidIdentifier, self.tabView.getTab, u'id5')
        self.assertNotIn(newTab, self.tabView.getGroup(u'group1').tabs)


    def test_removeTabs(self):
        """
        Removing tabs on the server side releases them and invokes methods on
        the client side to remove them.
        """
        self.result = None

        def callRemote(methodName, *a):
            self.result = methodName == '_removeTabsFromServer'

        self.patch(self.tabView, 'callRemote', callRemote)
        tab = self.tabView.tabs[0]
        self.tabView.removeTabs([tab])
        self.assertTrue(self.result)
        self.assertRaises(
            errors.InvalidIdentifier, self.tabView.getTab, tab.id)


    def test_invalidRemove(self):
        """
        Trying to remove an unmanaged tab results in C{ValueError} being
        raised.
        """
        # Try a new tab.
        self.assertRaises(ValueError,
            self.tabView.removeTabs, [
                widgets.Tab(u'id99', u'Title 1', self.contentFactory)])

        # Try dupe an ID.
        self.assertRaises(ValueError,
            self.tabView.removeTabs, [
                widgets.Tab(u'id1', u'Title 1', self.contentFactory)])


    def test_repr(self):
        """
        L{methanal.widgets.TabView} has an accurate string representation.
        """
        self.assertEquals(
            repr(self.tabView),
            "<TabView topLevel=False tabs=%r>" % self.tabs)



class StaticTabTests(unittest.TestCase):
    """
    Tests for L{methanal.widgets.StaticTab}.
    """
    def test_create(self):
        """
        Creating a L{methanal.widgets.StaticTab} instance, with the C{content}
        argument, wraps the content in a factory method. Passing a
        C{contentFactory} argument works as intended.
        """
        content = u'A content.'
        tab = widgets.StaticTab(
            id=u'id',
            title=u'Title',
            content=content)
        self.assertEquals(tab.contentFactory(), content)

        tab = widgets.StaticTab(
            id=u'id',
            title=u'Title',
            contentFactory=lambda: content * 2)
        self.assertEquals(tab.contentFactory(), content * 2)


    def test_repr(self):
        """
        L{methanal.widgets.Tab} (and by extension L{methanal.widgets.StaticTab})
        has an accurate string representation.
        """
        tab = widgets.StaticTab(
            id=u'id',
            title=u'Title',
            content=u'A content.')
        self.assertEquals(
            repr(tab),
            "<StaticTab id=u'id' title=u'Title' selected=False group=None>")




class TabGroupTests(unittest.TestCase):
    """
    Tests for L{methanal.widgets.TabGroup}.
    """
    def test_athenaSerializable(self):
        """
        Tab groups implement L{nevow.inevow.IAthenaTransportable}.
        """
        tabs = [
            widgets.Tab(u'id1', u'Title 1', None),
            widgets.Tab(u'id2', u'Title 2', None)]
        tabGroup = widgets.TabGroup(u'id', u'Title', tabs=tabs)
        self.assertEquals(
            inevow.IAthenaTransportable(tabGroup).getInitialArguments(),
            [u'id', u'Title', [u'id1', u'id2']])


    def test_mergeGroups(self):
        """
        L{methanal.widgets.TabGroup.mergeGroups} will merge two
        L{methanal.widgets.TabGroup}s together, preferring the metadata of the
        second specified group.
        """
        tabs = [
            widgets.Tab(u'id1', u'Title 1', None),
            widgets.Tab(u'id2', u'Title 2', None)]
        tabGroup1 = widgets.TabGroup(u'id', u'Title', tabs=tabs)
        tabs = [
            widgets.Tab(u'id3', u'Title 3', None)]
        tabGroup2 = widgets.TabGroup(u'id', u'Hello', tabs=tabs)

        newGroup = widgets.TabGroup.mergeGroups(tabGroup1, tabGroup2)
        self.assertEquals(newGroup.id, u'id')
        self.assertEquals(newGroup.title, u'Hello')
        self.assertEquals(newGroup.tabs, tabGroup1.tabs + tabGroup2.tabs)


    def test_repr(self):
        """
        L{methanal.widgets.TabGroup} has an accurate string representation.
        """
        tabs = [
            widgets.Tab(u'id1', u'Title 1', None),
            widgets.Tab(u'id2', u'Title 2', None)]
        tabGroup = widgets.TabGroup(u'id', u'Title', tabs=tabs)
        self.assertEquals(
            repr(tabGroup),
            "<TabGroup id=u'id' title=u'Title' tabs=%r>" % tabs)



class ComparableLiveElement(ThemedElement):
    """
    C{LiveElement} implementing C{__eq__} comparing C{self.value}.
    """
    def __init__(self, value):
        self.value = value


    def __eq__(self, other):
        try:
            return self.value == other.value
        except AttributeError:
            return False



class ExpanderTests(unittest.TestCase):
    """
    Tests for L{methanal.widgets.Expander}.
    """
    def test_getContent(self):
        """
        L{Expander.getContent} returns the content for the specified node ID,
        C{KeyError} is raised if an unknown node ID is specified.
        """
        expander = widgets.Expander(
            headerFactory=partial(ComparableLiveElement, u'foo'),
            contentFactory=partial(ComparableLiveElement, u'bar'))
        self.assertEqual(
            expander.getContent(u'header').value, u'foo')
        self.assertEqual(
            expander.getContent(u'content').value, u'bar')
        self.assertRaises(
            KeyError, expander.getContent, u'not_a_thing')


    def test_lazyHeaderContent(self):
        """
        L{Expander.getHeaderContent} returns the result of
        L{Expander.headerFactory} if C{currentContent == newContent} returns
        C{False}, otherwise C{None} is returned.
        """
        expander = widgets.Expander(
            headerFactory=partial(ComparableLiveElement, u'content'),
            contentFactory=None)

        # Fresh.
        content = expander.getHeaderContent()
        self.assertNotIdentical(None, content)
        self.assertEqual(u'content', content.value)
        # Same as last time
        self.assertIdentical(None, expander.getHeaderContent())
        # New value.
        expander.headerFactory = partial(ComparableLiveElement, u'other')
        content = expander.getHeaderContent()
        self.assertNotIdentical(None, content)
        self.assertEqual(u'other', content.value)


    def test_lazyExpanderContent(self):
        """
        L{Expander.getExpanderContent} returns the result of
        L{Expander.contentFactory} if C{currentContent == newContent} returns
        C{False}, otherwise C{None} is returned.
        """
        expander = widgets.Expander(
            headerFactory=None,
            contentFactory=partial(ComparableLiveElement, u'content'))

        # Fresh.
        content = expander.getExpanderContent()
        self.assertNotIdentical(None, content)
        self.assertEqual(u'content', content.value)
        # Same as last time
        self.assertIdentical(None, expander.getExpanderContent())
        # New value.
        expander.contentFactory = partial(ComparableLiveElement, u'other')
        content = expander.getExpanderContent()
        self.assertNotIdentical(None, content)
        self.assertEqual(u'other', content.value)



from fusion_util.errors import InvalidEnumItem
from twisted.python.deprecate import deprecatedModuleAttribute
from twisted.python.versions import Version



class ConstraintError(ValueError):
    """
    One or more constraints specified in the data model were unmet.
    """



class InvalidIdentifier(ValueError):
    """
    An invalid identifier was specified.
    """



deprecatedModuleAttribute(
    Version('Methanal', 0, 4, 0),
    'Use fusion_util.errors.InvalidEnumItem instead.',
    'methanal.errors',
    'InvalidEnumItem')



__all__ = ['InvalidEnumItem', 'ConstraintError', 'InvalidIdentifier']



"""
Public interfaces used in Methanal.
"""
from zope.interface import Interface, Attribute



class IColumn(Interface):
    """
    Represents a column that can be viewed via a query list, and provides hints
    and metadata about the column.
    """
    attributeID = Attribute(
        """
        An ASCII-encoded C{str} uniquely describing this column.
        """)


    title = Attribute(
        """
        C{unicode} text for a human-readable column title.
        """)


    def extractValue(model, item):
        """
        Extract a value for the column from an Item.

        @type model: L{methanal.widgets.Table}

        @type  item: L{axiom.item.Item}
        @param item: Item from which to extract a value

        @return: Underlying value for this column
        """


    def extractLink(model, item):
        """
        Extract a URI for the column from an Item.

        @type model: L{methanal.widgets.Table}

        @type  item: L{axiom.item.Item}
        @param item: Item from which to extract a URI

        @rtype: C{unicode}
        @return: A URI somehow relevant for this column, or C{None}
        """


    def getType():
        """
        Get a name identifying the type of data this column contains.

        @rtype: C{str}
        """



class IEnumeration(Interface):
    """
    An enumeration.
    """
    doc = Attribute("""
    A brief description of the enumeration's intent.
    """)

    def asPairs():
        """
        Represent the enumeration as a sequence of pairs.

        @rtype: C{list} of 2-C{tuple}s
        @return: A sequence of C{(value, description)}
        """


    def get(value):
        """
        Get an enumeration item for a given enumeration value.

        @raise L{methanal.errors.InvalidEnumItem}: If C{value} does not match
            any known enumeration value.

        @rtype: L{EnumItem}
        """


    def getDesc(value):
        """
        Get the description for a given enumeration value.
        """


    def getExtra(value, extraName, default=None):
        """
        Get the extra value for C{extraName} or use C{default}.
        """


    def find(**names):
        """
        Find the first L{EnumItem} with matching extra values.

        @param **names: Extra values to match

        @rtype:  L{EnumItem}
        @return: The first matching L{EnumItem} or C{None} if there are no
            matches
        """


    def findAll(**names):
        """
        Find all L{EnumItem}s with matching extra values.

        @param **names: Extra values to match

        @rtype:  C{iterable} of L{EnumItem}
        """



class ITextFormatter(Interface):
    """
    Format values as text in a human-readable way.
    """
    def format(value):
        """
        Provide a legible well-formatted representation of I{value}.

        For example: L{DecimalFormatter} performs digit grouping,
        L{CurrencyFormatter} applies a currency symbol and digit grouping.

        @rtype: C{unicode}
        """



from datetime import datetime
from decimal import Decimal
try:
    import xml.etree.ElementTree as etree
    # Appease pyflakes.
    etree
except ImportError:
    import elementtree.ElementTree as etree

from twisted.trial import unittest
from twisted.internet.defer import gatherResults

from epsilon.extime import FixedOffset, Time

from axiom import attributes
from axiom.store import Store
from axiom.item import Item

from nevow import athena, loaders
from nevow.testutil import renderLivePage

from methanal import view, errors
from methanal.imethanal import IEnumeration
from methanal.model import ItemModel, Value, DecimalValue, Model
from methanal.enums import Enum, EnumItem, ObjectEnum



class MockParent(object):
    def __init__(self, param):
        self.store = None
        self.param = param

        self.page = self
        self.liveFragmentChildren = []

        self.model = self


    def getParameter(self, name):
        return self.param


    def addFormChild(self, child):
        pass



def renderWidget(widget, debug=False):
    """
    Flatten and render a widget.

    @rtype: C{Deferred} -> C{ElementTree}
    @return: A deferred that fires with the parsed flattened output.
    """
    def parse(result):
        return etree.ElementTree(
            etree.fromstring('<widget>' + result + '</widget>'))

    def printIt(result):
        print result
        return result

    page = athena.LivePage(docFactory=loaders.stan(widget))
    widget.setFragmentParent(page)
    d = renderLivePage(page)
    if debug:
        d.addCallback(printIt)
    d.addCallback(parse)
    return d



class LiveFormTests(unittest.TestCase):
    """
    Tests for L{methanal.view.LiveForm}.
    """
    def setUp(self):
        self.form = view.LiveForm(store=None, model=Model())


    def test_renderActions(self):
        """
        The actions of a LiveForm are rendered according to the given actions.
        """
        def verifyRendering(tree):
            # Oh where, oh where has my little library gone?
            # Oh where, oh where can it be?
            # With real XPath support and functionality.
            # Oh where, oh where can it be?
            candidates = tree.findall('.//form/div')
            actionsNode = None
            id = 'athenaid:%d-actions' % (self.form._athenaID,)
            for node in candidates:
                if node.get('id') == id:
                    actionsNode = node
                    break

            self.assertNotIdentical(
                actionsNode, None, 'Could not find the form actions node')

            button = actionsNode.find('.//button')
            self.assertEquals(button.get('type'), 'submit')
            self.assertEquals(button.text.strip(), 'Submit')

        return renderWidget(self.form).addCallback(verifyRendering)



class FormInputTests(unittest.TestCase):
    """
    Tests for L{methanal.view.FormInput}.

    Also serves as the base class for other input tests.

    @type controlType: L{methanal.view.FormInput} subclass
    @cvar controlType: Type of control to create with L{createControl}

    @type createArgs: C{list} of C{dict}
    @cvar createArgs: Sequence of arguments that will create controls of
        type L{controlType}

    @type brokenCreateArgs: C{list} of C{(Exception, dict)}
    @cvar brokenCreateArgs: Sequence of C{(exceptionRaised, args)} that will
        fail to create controls of type L{controlType}
    """
    controlType = view.FormInput

    createArgs = []

    brokenCreateArgs = []


    def setUp(self):
        self.name = 'test'


    def createParent(self, args):
        """
        Create a parent control.
        """
        return MockParent(self.createParam(args))


    def createParam(self, args):
        """
        Create a model parameter.
        """
        return Value(self.name)


    def createControl(self, args):
        """
        Create a control of type L{controlType}.
        """
        args.setdefault('name', self.name)
        args.setdefault('parent', self.createParent(args))
        return self.controlType(**args)


    def test_creation(self):
        """
        Creating a control succeeds.

        Try each set of parameters from L{createArgs}.
        """
        for args in self.createArgs:
            control = self.createControl(args)
            self.assertTrue(isinstance(control.name, unicode))
            self.assertIdentical(control.param, control.parent.param)


    def test_creationFails(self):
        """
        Creating a control fails.

        Try each set of parameters from L{brokenCreateArgs}, ensuring that
        the correct exception is raised in each case.
        """
        for errorType, args in self.brokenCreateArgs:
            self.assertRaises(errorType, self.createControl, args)



class TextAreaInputTests(FormInputTests):
    """
    Tests for L{methanal.view.TextAreaInput}.
    """
    controlType = view.TextAreaInput

    createArgs = [
        dict()]



class TextInputTests(FormInputTests):
    """
    Tests for L{methanal.view.TextInput}.
    """
    controlType = view.TextInput

    createArgs = [
        dict(),
        dict(embeddedLabel=True)]



class FilteringTextInputTests(FormInputTests):
    """
    Tests for L{methanal.view.FilteringTextInput}.
    """
    controlType = view.FilteringTextInput

    createArgs = [
        dict(),
        dict(expression=u'[a-zA-Z]')]



class PrePopulatingTextInputTests(FormInputTests):
    """
    Tests for L{methanal.view.PrePopulatingTextInput}.
    """
    controlType = view.PrePopulatingTextInput

    createArgs = [
        dict(targetControlName='bbq')]

    brokenCreateArgs = [
        (TypeError, dict())]



class DateInputTests(FormInputTests):
    """
    Tests for L{methanal.view.DateInput}.
    """
    controlType = view.DateInput

    createArgs = [
        dict(timezone=FixedOffset(0, 0)),
        dict(timezone=FixedOffset(0, 0), twentyFourHours=True)]

    brokenCreateArgs = [
        (TypeError, dict()),
        (TypeError, dict(twentyFourHours=True))]


    def test_getValue(self):
        """
        L{methanal.view.DateInput.getValue} retrieves an empty string in the
        C{None} case and a string representing the C{Time} specified by
        parameter's value, in the case where a value exists.
        """
        control = self.createControl(dict(timezone=FixedOffset(0, 0)))
        param = control.parent.param

        param.value = None
        self.assertEquals(control.getValue(), u'')

        param.value = Time.fromDatetime(datetime(2007, 1, 1))
        self.assertTrue(isinstance(control.getValue(), unicode))
        self.assertEquals(control.getValue(), u'2007-01-01')

        param.value = Time.fromDatetime(datetime(542, 12, 18))
        self.assertEquals(control.getValue(), u'0542-12-18')



class NumericInputTestsMixin(object):
    """
    Tests mixin for numeric inputs.
    """
    def _testMinimumMaximum(self, control, acceptable, tooSmall, tooLarge):
        """
        Assert that C{control} sets its parameter's value to C{None} value or
        C{acceptable}, when invoked with these values. When invoked with
        C{tooSmall} or C{tooLarge} an exception is raised indicating that the
        value is too small or too large.
        """
        param = control.parent.param

        data = {param.name: None}
        control.invoke(data)
        self.assertIdentical(param.value, None)

        data = {param.name: acceptable}
        control.invoke(data)
        self.assertEquals(param.value, acceptable)

        data = {param.name: tooSmall}
        e = self.assertRaises(ValueError, control.invoke, data)
        self.assertIn('is smaller than', str(e))

        data = {param.name: tooLarge}
        e = self.assertRaises(ValueError, control.invoke, data)
        self.assertIn('is larger than', str(e))


    def test_minimumMaximumDefaultValues(self):
        """
        Numeric inputs default to not accepting values smaller than C{-(2 **
        63)} or larger than C{2 ** 63 }.
        """
        control = self.createControl(dict())
        self._testMinimumMaximum(control, 42, -(2 ** 63) - 1, 2 ** 63)


    def test_minimumMaximumOverrideValues(self):
        """
        Numeric inputs can override their minimum and maximum value range.
        """
        control = self.createControl(dict(minimumValue=0, maximumValue=10))
        self._testMinimumMaximum(control, 5, -1, 42)



class IntegerInputTests(FormInputTests, NumericInputTestsMixin):
    """
    Tests for L{methanal.view.IntegerInput}.
    """
    controlType = view.IntegerInput


    def test_getValue(self):
        """
        L{methanal.view.IntegerInput.getValue} retrieves an empty string in the
        C{None} case and an C{int} in the case where a value exists.
        """
        control = self.createControl(dict())
        param = control.parent.param

        param.value = None
        self.assertEquals(control.getValue(), u'')

        param.value = u'5'
        self.assertTrue(isinstance(control.getValue(), int))
        self.assertEquals(control.getValue(), 5)
        param.value = 5
        self.assertTrue(isinstance(control.getValue(), int))
        self.assertEquals(control.getValue(), 5)



class FloatInputTests(FormInputTests, NumericInputTestsMixin):
    """
    Tests for L{methanal.view.FloatInput}.
    """
    controlType = view.FloatInput


    def test_getValue(self):
        """
        L{methanal.view.FloatInput.getValue} retrieves an empty string in the
        C{None} case and a C{float} in the case where a value exists.
        """
        control = self.createControl(dict())
        param = control.parent.param

        param.value = None
        self.assertEquals(control.getValue(), u'')

        param.value = u'5'
        self.assertTrue(isinstance(control.getValue(), float))
        self.assertEquals(control.getValue(), 5.0)
        param.value = 5.0
        self.assertTrue(isinstance(control.getValue(), float))
        self.assertEquals(control.getValue(), 5.0)


    def test_minimumMaximumDefaultValues(self):
        """
        The limits for SQL and Python float types are the same value, so no
        minimum or maximum checking is required in Methanal for the default
        ranges.
        """
        pass



class DecimalInputTests(FormInputTests, NumericInputTestsMixin):
    """
    Tests for L{methanal.view.DecimalInput}.
    """
    controlType = view.DecimalInput

    decimalPlaces = 3

    createArgs = [
        dict(),
        dict(decimalPlaces=1)]


    def createParam(self, args):
        decimalPlaces = args.get('decimalPlaces', self.decimalPlaces)
        return DecimalValue(name=self.name, decimalPlaces=decimalPlaces)


    def test_getValue(self):
        """
        L{methanal.view.DecimalInput.getValue} retrieves an empty string in the
        C{None} case and a C{float} in the case where a value exists.
        """
        control = self.createControl(dict(decimalPlaces=2))
        param = control.parent.param

        param.value = None
        self.assertEquals(control.getValue(), u'')

        param.value = u'12.34'
        self.assertTrue(isinstance(control.getValue(), float))
        self.assertEquals(control.getValue(), 12.34)


    def test_invoke(self):
        """
        L{methanal.view.DecimalInput.invoke} sets the parameter value to C{None}
        in the C{None} case and a C{Decimal} in the case where a value exists.
        """
        control = self.createControl(dict(decimalPlaces=2))
        param = control.parent.param

        data = {param.name: None}
        control.invoke(data)
        self.assertIdentical(param.value, None)

        data = {param.name: u'12.34'}
        control.invoke(data)
        self.assertEquals(param.value, Decimal('12.34'))



class VerifiedPasswordInputTests(FormInputTests):
    """
    Tests for L{methanal.view.VerifiedPasswordInput}.
    """
    controlType = view.VerifiedPasswordInput

    createArgs = [
        dict(),
        dict(minPasswordLength=3),
        dict(strengthCriteria=['ALPHA'])]



class ChoiceInputTestsMixin(object):
    """
    Mixin for L{methanal.view.ChoiceInput} based controls.
    """
    def createControl(self, args):
        """
        Create a L{methanal.view.ChoiceInput} from C{values} and assert that
        L{methanal.view.ChoiceInput.values} provides L{IEnumeration}.
        """
        control = super(ChoiceInputTestsMixin, self).createControl(args)
        self.assertTrue(IEnumeration.providedBy(control.values))
        return control



class ObjectChoiceTestsMixin(ChoiceInputTestsMixin):
    """
    Mixin for L{methanal.view.ChoiceInput}s that support arbitrary Python
    objects.
    """
    values = [
        (object(), u'Foo'),
        (object(), u'Bar'),
        (None,     u'Baz')]

    createArgs = [
        dict(values=values)]

    def test_choiceValues(self):
        """
        L{ChoiceInput}s that support arbitrary Python objects maintain a
        mapping of object identities (stored as text) to objects, and use the
        object identites for enumeration values.
        """
        control = self.createControl(dict(values=self.values))
        expectedValues = [(id(obj), desc) for obj, desc in self.values]
        self.assertEquals(control.values.as_pairs(), expectedValues)


    def test_encodeDecodeValue(self):
        """
        L{_ObjectChoiceMixinBase.encodeValue} and
        L{_ObjectChoiceMixinBase.decodeValue} are symmetric.
        """
        control = self.createControl(dict(values=self.values))
        for obj, desc in self.values:
            self.assertIdentical(
                control.decodeValue(control.encodeValue(obj)), obj)


    def test_getValue(self):
        """
        Object-enabled L{ChoiceInput}s return an encoded object identity from
        their C{getValue} method.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param

        for obj, desc in self.values:
            param.value = obj
            self.assertEquals(
                control.getValue(),
                control.encodeValue(obj))


    def test_invoke(self):
        """
        Object-enabled L{ChoiceInput}s set their parameter value to the
        corresponding real object when invoked with an encoded object identity.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param

        pairs = [(obj, value)
                 for (obj, desc), (value, desc)
                 in zip(self.values, control.values.as_pairs())]
        for obj, value in pairs:
            data = {param.name: value}
            control.invoke(data)
            self.assertIdentical(param.value, obj)


    def test_render(self):
        """
        Rendering the control raises no exceptions.
        """
        control = self.createControl(dict(values=self.values))
        ds = []

        def _render(value):
            control.parent.param.value = value
            ds.append(renderWidget(control))

        _render(None)
        for obj, desc in self.values:
            _render(obj)
        return gatherResults(ds)



class ObjectMultiChoiceTestsMixin(ObjectChoiceTestsMixin):
    """
    Mixin for L{methanal.view.ChoiceInput}s that support many arbitrary Python
    objects.
    """
    def test_getValueNoValues(self):
        """
        Multiple-object-enabled L{ChoiceInput}s return an empty list from
        C{getValue} when the parameter is C{None}, or empty.
        """
        control = self.createControl(dict(values=self.values))

        control.parent.param.value = None
        self.assertEquals(control.getValue(), [])

        control.parent.param.value = []
        self.assertEquals(control.getValue(), [])


    def test_getValue(self):
        """
        Multiple-object-enabled L{ChoiceInput}s return many encoded object
        identities from C{getValue}.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param

        for obj, desc in self.values:
            param.value = [obj]
            self.assertEquals(
                control.getValue(),
                [control.encodeValue(obj)])

        obj1, obj2 = [self.values[0][0], self.values[1][0]]
        param.value = [obj1, obj2]
        self.assertEquals(
            control.getValue(),
            [control.encodeValue(obj1), control.encodeValue(obj2)])


    def test_invokeNoValues(self):
        """
        Multiple-object-enabled L{ChoiceInput}s set their parameter value to an
        empty list when invoked with C{None}, or empty.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param
        control.invoke({param.name: None})
        self.assertEquals(param.value, [])

        control = self.createControl(dict(values=self.values))
        param = control.parent.param
        control.invoke({param.name: []})
        self.assertEquals(param.value, [])


    def test_invoke(self):
        """
        Multiple-object-enabled L{ChoiceInput}s set their parameter value to
        many corresponding real objects when invoked with encoded object
        identities.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param

        pairs = [(obj, value)
                 for (obj, desc), (value, desc)
                 in zip(self.values, control.values.as_pairs())]
        for obj, value in pairs:
            data = {param.name: [value]}
            control.invoke(data)
            self.assertEquals(param.value, [obj])

        obj1, obj2 = [pairs[0][0], pairs[1][0]]
        value1, value2 = [pairs[0][1], pairs[1][1]]
        data = {param.name: [value1, value2]}
        control.invoke(data)
        self.assertEquals(param.value, [obj1, obj2])


    def test_render(self):
        """
        Rendering the control raises no exceptions.
        """
        control = self.createControl(dict(values=self.values))
        ds = []

        def _render(value):
            control.parent.param.value = value
            ds.append(renderWidget(control))

        _render(None)
        for obj, desc in self.values:
            _render([obj])
        return gatherResults(ds)



class ChoiceInputTests(ChoiceInputTestsMixin, FormInputTests):
    """
    Tests for L{methanal.view.ChoiceInput}.
    """
    controlType = view.ChoiceInput

    createArgs = [
        dict(values=[
            (u'foo', u'Foo'),
            (u'bar', u'Bar')])]

    brokenCreateArgs = [
        (TypeError, dict())]


    def createControl(self, args, checkExpectedValues=True):
        """
        Create a L{methanal.view.ChoiceInput} from C{values}, assert that
        L{methanal.view.ChoiceInput.value} provides L{IEnumeration} and
        calling C{as_pairs} results in the same values as C{values}.
        """
        control = super(ChoiceInputTests, self).createControl(args)
        if checkExpectedValues:
            self.assertEquals(control.values.as_pairs(),
                              list(args.get('values')))
        return control


    def test_createDeprecated(self):
        """
        Passing values that are not adaptable to IEnumeration are converted
        to a C{list}, adapted to L{IEnumeration} and a warning is emitted.
        """
        # Not a list.
        values = tuple([
            (u'foo', u'Foo'),
            (u'bar', u'Bar')])
        self.createControl(dict(values=values))
        self.assertEquals(len(self.flushWarnings()), 2)


    def test_invalidEnumValues(self):
        """
        Specifying invalid enumeration item values returns C{None} instead of
        raising L{InvalidEnumItem}.
        """
        values = [
            (u'foo', u'Foo'),
            (u'bar', u'Bar')]
        control = self.createControl(dict(values=values))
        control.param.value = u'nopenotinhere'
        self.assertEqual(
            control.getValue(),
            None)



class SelectInputTests(ChoiceInputTests):
    """
    Tests for L{methanal.view.SelectInput}.
    """
    controlType = view.SelectInput

    def test_renderGroupsOptions(self):
        """
        The options of a C{SelectInput} are rendered as groups if the enumeration
        values specify a C{'group'} extra value.
        """
        values = Enum(
            '',
            [EnumItem(u'foo', u'Foo', group=u'Group'),
             EnumItem(u'bar', u'Bar', group=u'Group')])

        control = self.createControl(
            dict(values=values), checkExpectedValues=False)

        def verifyRendering(tree):
            groupNodes = tree.findall('.//select/optgroup')
            groups = set(item.group for item in values)
            self.assertEquals(len(groupNodes), len(groups))
            for groupNode, item in zip(groupNodes, values):
                self.assertEquals(groupNode.get('label'), item.group)

                optionNodes = groupNode.findall('option')
                self.assertEquals(len(optionNodes), len(list(values)))
                for optionNode, innerItem in zip(optionNodes, values):
                    self.assertEquals(optionNode.get('value'), innerItem.value)
                    self.assertEquals(optionNode.text.strip(), innerItem.desc)

        return renderWidget(control).addCallback(verifyRendering)


    def test_renderObjectOptions(self):
        """
        If a C{SelectInput}'s values are an C{ObjectEnum} then the rendered
        option values match the C{'id'} extra of the object enumeration.
        """
        object1 = object()
        object2 = object()
        values = ObjectEnum(
            '',
            [EnumItem(object1, u'Foo'),
             EnumItem(object2, u'Bar', id=u'chuck')])

        control = self.createControl(
            dict(values=values), checkExpectedValues=False)

        def verifyRendering(tree):
            optionNodes = tree.findall('.//option')
            self.assertEquals(len(optionNodes), len(list(values)))
            for optionNode, item in zip(optionNodes, values):
                self.assertEquals(optionNode.get('value'), item.id)
                self.assertEquals(optionNode.text.strip(), item.desc)

        return renderWidget(control).addCallback(verifyRendering)


    def test_getValueObjectEnum(self):
        """
        L{SelectInput.getValue} returns the C{'id'} extra value when backed by
        an C{ObjectEnum}.
        """
        object1 = object()
        object2 = object()
        values = ObjectEnum(
            '',
            [EnumItem(object1, u'Foo'),
             EnumItem(object2, u'Bar', id=u'chuck')])

        control = self.createControl(
            dict(values=values), checkExpectedValues=False)

        param = control.parent.param

        param.value = None
        self.assertIdentical(control.getValue(), None)

        for value in values:
            param.value = value.value
            self.assertEquals(control.getValue(), value.id)


    def test_invokeObjectEnum(self):
        """
        When backed by an C{ObjectEnum}, L{SelectInput.invoke} sets the
        parameter value to the Python object of L{EnumItem} with C{'id'} extra
        matching the invocation data value and C{None} in the C{None} case.
        """
        object1 = object()
        object2 = object()
        values = ObjectEnum(
            '',
            [EnumItem(object1, u'Foo'),
             EnumItem(object2, u'Bar', id=u'chuck')])

        control = self.createControl(
            dict(values=values), checkExpectedValues=False)

        param = control.parent.param
        data = {param.name: None}
        control.invoke(data)
        self.assertIdentical(param.value, None)

        for value in values:
            data = {param.name: value.id}
            control.invoke(data)
            self.assertIdentical(param.value, value.value)


    def test_invokeObjectEnumTrickery(self):
        """
        L{SelectInput.invoke} falls back to locating enumeration items by value
        when no C{'id'} extra value matches. However, this can lead to leaking
        information about the enumeration values in the case of objects such as
        strings and integers. In case we fallback to an item with a C{'id'}
        extra, even if it doesn't match, C{InvalidEnumItem} is raised.
        """
        object1 = object()
        values = ObjectEnum(
            '',
            [EnumItem(object1,     u'Foo'),
             EnumItem(u'password', u'Bar', id=u'chuck')])

        control = self.createControl(
            dict(values=values), checkExpectedValues=False)

        param = control.parent.param

        data = {param.name: u'password'}
        self.assertRaises(errors.InvalidEnumItem,
            control.invoke, data)



class MultiValueChoiceInputTestsMixin(object):
    """
    Tests mixin for L{methanal.view.ChoiceInput}s that support multiple values.
    """
    values = [
        (u'foo', u'Foo'),
        (u'bar', u'Bar'),
        (u'baz', u'Baz')]

    def test_getValueNoValues(self):
        """
        Multi-value L{ChoiceInput}s return an empty list from C{getValue} when
        the parameter is C{None}, or empty.
        """
        control = self.createControl(dict(values=self.values))

        control.parent.param.value = None
        self.assertEquals(control.getValue(), [])

        control.parent.param.value = []
        self.assertEquals(control.getValue(), [])


    def test_getValue(self):
        """
        Multi-value L{ChoiceInput}s return a C{list} of the selected values
        from C{getValue}.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param

        for value, desc in self.values:
            param.value = [value]
            self.assertEquals(
                control.getValue(),
                [value])

        v1, v2 = [self.values[0][0], self.values[1][0]]
        param.value = [v1, v2]
        self.assertEquals(
            control.getValue(),
            [v1, v2])


    def test_invokeNoValues(self):
        """
        Multi-value L{ChoiceInput}s set their parameter value to an empty list
        when invoked with C{None}, or empty.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param
        control.invoke({param.name: None})
        self.assertEquals(param.value, [])

        control = self.createControl(dict(values=self.values))
        param = control.parent.param
        control.invoke({param.name: []})
        self.assertEquals(param.value, [])


    def test_invoke(self):
        """
        Multi-value L{ChoiceInput}s set their parameter value to a C{list}
        containing the values of the invoked data.
        """
        control = self.createControl(dict(values=self.values))
        param = control.parent.param

        data = {param.name: None}
        control.invoke(data)
        self.assertEquals(param.value, [])

        data = {param.name: [None]}
        control.invoke(data)
        self.assertEquals(param.value, [None])

        for value, desc in self.values:
            data = {param.name: [value]}
            control.invoke(data)
            self.assertEquals(param.value, [value])

        v1, v2 = self.values[0][0], self.values[1][0]
        data = {param.name: [v1, v2]}
        control.invoke(data)
        self.assertEquals(param.value, [v1, v2])


    def test_invokeObjectEnum(self):
        """
        Invoking a multi-value L{ChoiceInput}s backed by an L{ObjectEnum} sets
        their parameter value to a C{list} of objects referenced by ID in the
        invocation data.
        """
        object1 = object()
        values = ObjectEnum(
            '',
            [EnumItem(object1, u'Foo'),
             EnumItem(u'buzz', u'Buzz', id=u'quux')])
        control = self.createControl(
            dict(values=values), checkExpectedValues=False)
        param = control.parent.param

        for value in values:
            data = {param.name: [value.id]}
            control.invoke(data)
            self.assertEquals(param.value, [value.value])

        vs = [v.value for v in values]
        vIDs = [v.id for v in values]
        data = {param.name: vIDs}
        control.invoke(data)
        self.assertEquals(param.value, vs)

        data = {param.name: [u'buzz']}
        self.assertRaises(errors.InvalidEnumItem, control.invoke, data)


    def test_render(self):
        """
        Rendering the control raises no exceptions.
        """
        control = self.createControl(dict(values=self.values))
        ds = []

        def _render(value):
            control.parent.param.value = value
            ds.append(renderWidget(control))

        _render(None)
        for obj, desc in self.values:
            _render([obj])
        return gatherResults(ds)


    def test_invalidEnumValues(self):
        """
        Invalid enumeration item values are stripped instead of raising
        L{InvalidEnumItem}.
        """
        values = [
            (u'foo', u'Foo'),
            (u'bar', u'Bar')]
        control = self.createControl(dict(values=values))
        control.param.value = [u'nopenotinhere']
        self.assertEqual(
            control.getValue(),
            [])

        control.param.value = [u'nopenotinhere', u'foo']
        self.assertEqual(
            control.getValue(),
            [u'foo'])



class MultiSelectInputTests(MultiValueChoiceInputTestsMixin, ChoiceInputTests):
    controlType = view.MultiSelectInput



class GroupedSelectInputTests(ChoiceInputTests):
    """
    Tests for L{methanal.view.GroupedSelectInput}.
    """
    controlType = view.GroupedSelectInput

    def test_createDeprecated(self):
        """
        Passing values that are not adaptable to IEnumeration are converted
        to a C{list}, adapted to L{IEnumeration} and a warning is emitted.
        """
        # Not a list.
        values = tuple([
            (u'foo', u'Foo'),
            (u'bar', u'Bar')])
        self.createControl(dict(values=values))
        self.assertEquals(len(self.flushWarnings()), 3)


    def test_renderOptions(self):
        """
        The options of a GroupedSelectInput are rendered according to the given
        values.
        """
        values = [(u'Group', [(u'foo', u'Foo'),
                              (u'bar', u'Bar')])]

        control = self.createControl(
            dict(values=values), checkExpectedValues=False)

        def verifyRendering(tree):
            groupNodes = tree.findall('.//select/optgroup')
            self.assertEquals(len(groupNodes), len(values))
            for groupNode, (group, subvalues) in zip(groupNodes, values):
                self.assertEquals(groupNode.get('label'), group)

                optionNodes = groupNode.findall('option')
                self.assertEquals(len(optionNodes), len(subvalues))
                for optionNode, (value, desc) in zip(optionNodes, subvalues):
                    self.assertEquals(optionNode.get('value'), value)
                    self.assertEquals(optionNode.text.strip(), desc)

        return renderWidget(control).addCallback(verifyRendering)



class ObjectGroupedSelectInputTests(ObjectChoiceTestsMixin, FormInputTests):
    """
    Tests for L{methanal.view.ObjectGroupedSelectInput}.
    """
    controlType = view.ObjectGroupedSelectInput



class ObjectSelectInputTests(ObjectChoiceTestsMixin, FormInputTests):
    """
    Tests for L{methanal.view.ObjectSelectInput}.
    """
    controlType = view.ObjectSelectInput



class IntegerSelectInputTests(ObjectSelectInputTests):
    """
    Tests for L{methanal.view.IntegerSelectInput}.
    """
    controlType = view.IntegerSelectInput

    values = [
        (1, u'Foo'),
        (2, u'Bar')]



class ObjectMultiSelectInputTests(ObjectMultiChoiceTestsMixin, FormInputTests):
    """
    Tests for L{methanal.view.ObjectMultiSelectInput}.
    """
    controlType = view.ObjectMultiSelectInput



class RadioGroupInputTests(ChoiceInputTests):
    """
    Tests for L{methanal.view.RadioGroupInput}.
    """
    controlType = view.RadioGroupInput


    def test_renderOptions(self):
        """
        The options of a GroupedSelectInput are rendered according to the given
        values.
        """
        values = [(u'foo', u'Foo'),
                  (u'bar', u'Bar')]

        control = self.createControl(dict(values=values))

        def verifyRendering(tree):
            inputs = tree.findall('.//input')
            self.assertEquals(len(values), len(inputs))
            for input, (value, desc) in zip(inputs, values):
                self.assertEquals(input.get('name'), self.name)
                self.assertEquals(input.get('value'), value)
                self.assertEquals(input.tail.strip(), desc)

        return renderWidget(control).addCallback(verifyRendering)



class ObjectRadioGroupInputTests(ObjectChoiceTestsMixin, FormInputTests):
    """
    Tests for L{methanal.view.ObjectRadioGroupInput}.
    """
    controlType = view.ObjectRadioGroupInput



class CheckboxInputTests(FormInputTests):
    """
    Tests for L{methanal.view.CheckboxInput}.
    """
    controlType = view.CheckboxInput


    def test_renderNotChecked(self):
        """
        An unchecked CheckboxInput renders no C{checked} attribute.
        """
        control = self.createControl(dict())
        control.parent.param.value = False

        def verifyRendering(tree):
            input = tree.find('.//input')
            self.assertIdentical(input.get('checked'), None)

        return renderWidget(control).addCallback(verifyRendering)


    def test_renderChecked(self):
        """
        An checked CheckboxInput renders a C{checked} attribute.
        """
        control = self.createControl(dict())
        control.parent.param.value = True

        def verifyRendering(tree):
            input = tree.find('.//input')
            self.assertEquals(input.get('checked'), 'checked')

        return renderWidget(control).addCallback(verifyRendering)


    def test_renderInlineLabel(self):
        """
        Specifying an C{inlineLabel} value results in that label being used
        when renderering the CheckboxInput.
        """
        control = self.createControl(dict(inlineLabel=u'HELLO WORLD'))
        control.parent.param.value = True

        def verifyRendering(tree):
            input = tree.find('.//input')
            self.assertEquals(input.tail.strip(), 'HELLO WORLD')

        return renderWidget(control).addCallback(verifyRendering)



class MultiCheckboxInputTests(MultiValueChoiceInputTestsMixin,
                              ChoiceInputTests):
    controlType = view.MultiCheckboxInput



class TestItem(Item):
    """
    A test Item with some attributes.
    """
    foo = attributes.text(doc=u'Foo')
    bar = attributes.boolean(doc=u'Bar?')



class ItemViewBaseTests(unittest.TestCase):
    """
    Tests for L{methanal.view.ItemViewBase}.
    """
    viewType = view.ItemViewBase

    def setUp(self):
        self.store = Store()
        self.item = TestItem(store=self.store)


    def test_createWithItem(self):
        """
        Create an item view with an axiom Item instance.
        """
        iv = self.viewType(item=self.item)
        self.assertIdentical(iv.store, self.store)
        self.assertIdentical(iv.itemClass, type(self.item))
        self.assertIdentical(iv.original, self.item)


    def test_createWithItemClass(self):
        """
        Create an item view with an axiom Item type and store, an exception
        is raised if the C{store} parameter is not given.
        """
        iv = self.viewType(itemClass=type(self.item), store=self.store)
        self.assertIdentical(iv.store, self.store)
        self.assertIdentical(iv.itemClass, type(self.item))
        self.assertIdentical(iv.original, type(self.item))

        self.assertRaises(ValueError, self.viewType, itemClass=type(self.item))


    def test_customModelFactory(self):
        """
        Creating an item view subclass with a custom model factory.
        """
        class TestModel(ItemModel):
            pass

        class CustomItemView(self.viewType):
            modelFactory = TestModel

        iv = CustomItemView(item=self.item)
        self.assertTrue(isinstance(iv.model, TestModel))



class ItemViewTests(ItemViewBaseTests):
    """
    Tests for L{methanal.view.ItemView}.
    """
    viewType = view.ItemView

    def test_invoke(self):
        """
        Invoking an item view will set the attributes on the underlying item to
        the invoked values for the available inputs.
        """
        iv = self.viewType(item=self.item)
        # Without inputs, invoking the item view won't do anything.
        view.TextInput(parent=iv, name='foo')
        view.CheckboxInput(parent=iv, name='bar')
        iv.invoke({'foo': u'hello',
                   'bar': False})

        item = iv.item
        self.assertIdentical(item, self.item)
        self.assertEquals(item.foo, u'hello')
        self.assertEquals(item.bar, False)


    def test_invokeWithItemClass(self):
        """
        Invoking an item view, without a concrete underlying item, will create
        the item, with the invoked values, when invoked.
        """
        iv = self.viewType(
            store=self.store, itemClass=type(self.item), switchInPlace=True)
        self.assertIdentical(iv.item, None)
        # Without inputs, invoking the item view won't do anything.
        view.TextInput(parent=iv, name='foo')
        view.CheckboxInput(parent=iv, name='bar')
        iv.invoke({'foo': u'hello',
                   'bar': False})

        item = iv.item
        self.assertNotIdentical(item, None)
        self.assertEquals(item.foo, u'hello')
        self.assertEquals(item.bar, False)



class AutoItemViewTests(ItemViewTests):
    """
    Tests for L{methanal.view.AutoItemView}.
    """
    viewType = view.AutoItemView
